#include <iostream>
#include <algorithm>
#include <vector>

using namespace std;

struct Tree {
	struct Seg {
		int left, right;
		Seg(int _left, int _right) : left(_left), right(_right) {}
		int size() { return right - left; }
		int mid() { return (left + right) / 2; }
		Seg leftMid() { return { left, mid() }; }
		Seg midRight() { return { mid(), right }; }
		bool have(int i) { return left <= i && i < right; }
		bool inside(Seg seg) { return seg.have(left) && seg.have(right - 1); }
	};
	bool cross(Seg seg1, Seg seg2) {
		return seg1.have(seg2.left) || seg2.have(seg1.left);
	}
	int n;
	vector<int> tree, upd;
	Tree(int _n) : n(_n) { resize(); }
	void resize() { tree.resize(4 * n); upd.resize(4 * n, -1); }
	void push(int pos, Seg cur) {
		if (upd[pos] == -1) return;
		if (cur.size() == 1) {
			tree[pos] = upd[pos];
			upd[pos] = -1;
		}
		else {
			upd[2 * pos + 1] = upd[pos];
			upd[2 * pos + 2] = upd[pos];
			upd[pos] = -1;
		}
	}
	void updateSeg(int pos, Seg cur, Seg updSeg, int newVal) {
		push(pos, cur);
		if (!cross(cur, updSeg)) return;
		if (cur.inside(updSeg)) {
			upd[pos] = newVal;
			push(pos, cur);
			return;
		}
		updateSeg(2 * pos + 1, cur.leftMid(), updSeg, newVal);
		updateSeg(2 * pos + 2, cur.midRight(), updSeg, newVal);
	}
	int get(int pos, Seg cur, int i) {
		push(pos, cur);
		if (!cur.have(i)) return -1;
		if (cur.size() == 1) return tree[pos];
		return max(get(2 * pos + 1, cur.leftMid(), i), get(2 * pos + 2, cur.midRight(), i));
	}
	int operator[](int i) { return get(0, { 0, n }, i); }
};

struct Item {
	int c, L, R;
	Item(int _c, int _L, int _R) : c(_c), L(_L), R(_R) {}
};
ostream& operator<<(ostream &out, Item it) { return out << it.c + 1 << " " << it.L + 1 << " " << it.R + 1; }

bool check(vector<int> &a, vector<Item> &ans, int n, int m) {
	Tree tree(m);
	for (auto elem : ans) {
		tree.updateSeg(0, { 0, m }, { elem.L, elem.R + 1 }, elem.c);
	}
	for (int i = 0; i < m; ++i) {
		if (tree[i] != a[i]) {
			cout << i << endl;
			return true;
		}
	}
	return false;
}

int main() {
	ios_base::sync_with_stdio(false); cin.tie(NULL); cout.tie(NULL);
	int n, m;
	cin >> m >> n;
	vector<int> a(m);
	for (int i = 0; i < m; ++i) cin >> a[i], --a[i];
	vector<int> last(n, -1);
	for (int i = 0; i < m; ++i) last[a[i]] = i;
	vector<int> first(n, -1);
	for (int i = m - 1; i >= 0; --i) first[a[i]] = i;
	vector<Item> ans;
	for (int i = 0; i < m; ++i) {
		if (first[a[i]] == i) {
			ans.push_back({ a[i], i, last[a[i]] });
		}
	}
	if (check(a, ans, n, m)) {
		cout << -1 << '\n';
	}
	else {
		cout << ans.size() << '\n';
		for (auto elem : ans) {
			cout << elem << '\n';
		}
	}
	system("pause");
	return 0;
}