#include <iostream>
#include <cmath>
#include <algorithm>
#include <vector>
#include <cassert>

using namespace std;

struct Tree {
	struct Seg {
		int left, right;
		Seg(int _left, int _right) : left(_left), right(_right) {}
		int size() { return right - left; }
		int mid() { return (left + right) / 2; }
		Seg leftMid() { return { left, mid() }; }
		Seg midRight() { return { mid(), right }; }
		bool have(int i) { return left <= i && i < right; }
		bool inside(Seg seg) { return seg.have(left) && seg.have(right - 1); }
	};
	bool cross(Seg seg1, Seg seg2) {
		return seg1.have(seg2.left) || seg2.have(seg1.left);
	}
	int n;
	vector<int> tree;
	Tree(int _n) : n(_n) { resize(); }
	void resize() { tree.resize(4 * n); }
	int updElem(int pos, Seg cur, int i, int newVal) {
		if (!cur.have(i)) return tree[pos];
		if (cur.size() == 1) return tree[pos] = newVal;
		return tree[pos] = max(updElem(2 * pos + 1, cur.leftMid(), i, newVal), updElem(2 * pos + 2, cur.midRight(), i, newVal));
	}
	int getMinInd(int pos, Seg cur, Seg q, int x) {
		if (!cross(cur, q)) return n;
		if (cur.size() == 1) {
			if (tree[pos] >= x) return cur.left;
			else return n;
		}
		if (cur.inside(q)) {
			if (tree[2 * pos + 1] >= x) return getMinInd(2 * pos + 1, cur.leftMid(), q, x);
			else if (tree[2 * pos + 2] >= x) return getMinInd(2 * pos + 2, cur.midRight(), q, x);
			else return n;
		}
		return min(getMinInd(2 * pos + 1, cur.leftMid(), q, x), getMinInd(2 * pos + 2, cur.midRight(), q, x));
	}
	int getMaxInd(int pos, Seg cur, Seg q, int x) {
		if (!cross(cur, q)) return -1;
		if (cur.size() == 1) {
			if (tree[pos] >= x) return cur.left;
			else return -1;
		}
		if (cur.inside(q)) {
			if (tree[2 * pos + 2] >= x) return getMaxInd(2 * pos + 2, cur.midRight(), q, x);
			else if (tree[2 * pos + 1] >= x) return getMaxInd(2 * pos + 1, cur.leftMid(), q, x);
			else return -1;
		}
		return max(getMaxInd(2 * pos + 1, cur.leftMid(), q, x), getMaxInd(2 * pos + 2, cur.midRight(), q, x));
	}
	int getMax(int pos, Seg cur, Seg q) {
		if (!cross(cur, q)) return -1e9;
		if (cur.inside(q)) return tree[pos];
		return max(getMax(2 * pos + 1, cur.leftMid(), q), getMax(2 * pos + 2, cur.midRight(), q));
	}
};

void test() {
	Tree tree(10);
	tree.updElem(0, { 0, 10 }, 1, -5);
	tree.updElem(0, { 0, 10 }, 3, 1);
	tree.updElem(0, { 0, 10 }, 2, -1);
	tree.updElem(0, { 0, 10 }, 7, 9);
	tree.updElem(0, { 0, 10 }, 3, 4);
	cout << tree.getMax(0, { 0, 10 }, {0, 1}) << endl;
	cout << tree.getMax(0, { 0, 10 }, {0, 10}) << endl;
	cout << tree.getMax(0, { 0, 10 }, {0, 15}) << endl;
	cout << tree.getMax(0, { 0, 10 }, {2, 5}) << endl;
	cout << tree.getMax(0, { 0, 10 }, {2, 10}) << endl;
	cout << endl;
	cout << tree.getMinInd(0, { 0, 10 }, { 2, 4 }, 0) << endl;
	cout << tree.getMinInd(0, { 0, 10 }, { 2, 4 }, -1) << endl;
}

int main() {
	//test();
	ios_base::sync_with_stdio(false); cin.tie(NULL); cout.tie(NULL);
	int n, m, q;
	cin >> n >> m >> q;
	vector<int> tr(n), tc(m);
	Tree trtr(n), trtrm(n), trtc(m), trtcm(m);
	for (int t = 1; t <= q; ++t) {
		int type; cin >> type;
		if (type == 1) {
			int r; cin >> r;
			tr[--r] = t;
			trtr.updElem(0, { 0, n }, r, t);
			trtrm.updElem(0, { 0, n }, r, -t);
		}
		else if (type == 2) {
			int c; cin >> c;
			tc[--c] = t;
			trtc.updElem(0, { 0, m }, c, t);
			trtcm.updElem(0, { 0, m }, c, -t);
		}
		else {
			int a, b, x, y, k;
			cin >> a >> b >> x >> y >> k;
			--a; --b; --x; --y;
			int sr = t - tr[a];
			int sc = t - tc[b];
			int fr = t - tr[x];
			int fc = t - tc[y];
			if ((sr > k && sc > k) || (fr > k && fc > k)) {
				cout << -1 << '\n';
			}
			else if ((sr <= k && fc <= k) || (sc <= k && fr <= k)) {
				cout << abs(a - x) + abs(b - y) << '\n';
			}
			else if ((a == x && sr <= k) || (b == y && sc <= k)) {
				cout << abs(a - x) + abs(b - y) << '\n';
			}
			else {
				int ans = 1e9;
				if (a > x) swap(a, x);
				if (b > y) swap(b, y);
				if (sr <= k) {
					int c1 = trtc.getMaxInd(0, { 0, m }, { 0, y + 1 }, t - k);
					int c2 = trtc.getMinInd(0, { 0, m }, { y + 1, m + 1 }, t - k);
					if (c1 != -1) ans = min(ans, abs(a - x) + abs(b - c1) + abs(y - c1));
					if (c2 != m) ans = min(ans, abs(a - x) + abs(b - c2) + abs(y - c2));

					if (t + trtrm.getMax(0, { 0, n }, { a, x + 1 }) <= k) {
						ans = abs(a - x) + abs(b - y);
					}
				}
				else {
					int r1 = trtr.getMaxInd(0, { 0, n }, { 0, x + 1 }, t - k);
					int r2 = trtr.getMinInd(0, { 0, n }, { x + 1, n + 1 }, t - k);
					if (r1 != -1) ans = min(ans, abs(b - y) + abs(a - r1) + abs(x - r1));
					if (r2 != n) ans = min(ans, abs(b - y) + abs(a - r2) + abs(x - r2));

					if (t + trtcm.getMax(0, { 0, m }, { b, y + 1 }) <= k) {
						ans = abs(a - x) + abs(b - y);
					}
				}
				//assert(ans != 1e9);
				cout << (ans == 1e9 ? -1 : ans) << '\n';
			}
		}
	}
	system("pause");
	return 0;
}
/*
4 3 9
3 2 1 3 3 1
3 2 1 3 3 1
2 1
2 3
1 1
3 2 1 3 3 3
3 2 1 3 3 3
2 2
3 2 1 3 3 6
*/