//#include "pch.h"
#include <iostream>
#include <vector>
#include <string>
#include <memory>

using namespace std;

#define int long long

struct Node {
	int x, y;
	shared_ptr<Node> l, r;

	Node() {}

	Node(int x) : x(x), y(rand()) {}
};

using NH = shared_ptr<Node>;

pair<NH, NH> split(NH t, int k) {
	if (!t) {
		return { 0, 0 };
	}
	if (t->x < k) {
		auto p = split(t->r, k);
		t->r = p.first;
		return { t, p.second };
	}
	else {
		auto p = split(t->l, k);
		t->l = p.second;
		return { p.first, t };
	}
}

NH merge(NH t1, NH t2) {
	if (!t2) return t1;
	if (!t1) return t2;
	if (t1->y > t2->y) {
		t1->r = merge(t1->r, t2);
		return t1;
	}
	else {
		t2->l = merge(t1, t2->l);
		return t2;
	}
}

bool contains(NH t, int a) {
	if (!t) {
		return false;
	}
	if (t->x == a) {
		return true;
	}
	if (t->x > a) {
		return contains(t->l, a);
	}
	else {
		return contains(t->r, a);
	}
}

void add(NH &t, int a) {
	if (contains(t, a)) {
		return;
	}
	auto p = split(t, a);
	NH b = make_shared<Node>(a);
	t = merge(merge(p.first, b), p.second);
}

int left(NH t) {
	if (!t) return -10;
	if (!t->l) {
		return t->x;
	}
	return left(t->l);
}

int find(NH t, int a) {
	auto p = split(t, a);
	int x = left(p.second);
	t = merge(p.first, p.second);
	return x;
}

void out(NH t)
{
	if (!t) return;
	cout << t->x << " ";
	out(t->l);
	out(t->r);
}

vector<vector<pair<int, int>>> g;
vector<bool> used;
vector<NH> paths;

void search(NH t, pair<int, int> v) {
	if (t->r) search(t->r, v);
	if (t->l) search(t->l, v);
	add(paths[v.first], t->x + v.second);
}

void dfs(int u) {
	used[u] = true;
	for (auto v : g[u]) {
		search(paths[u], v);
		dfs(v.first);
	}
}

string solve() {
	string ans = "";
	int n, m, q, p;
	cin >> n >> m >> q >> p;
	g = vector<vector<pair<int, int>>>(n);
	for (int i = 0; i < m; ++i) {
		int u, v, w;
		cin >> u >> v >> w;
		u--;
		v--;
		g[u].push_back({ v, w });
	}
	used = vector<bool>(n, false);
	paths = vector<NH>(n);
	add(paths[0], 0);
	dfs(0);
	for (int i = 0; i < q; ++i) {
		int u, r;
		cin >> u >> r;
		u--;
		int len = find(paths[u], r);
		if (r <= len && len <= (double)p / (p - 1) * r) {
			ans += "1";
		}
		else {
			ans += "0";
		}
	}
	for (int i = 0; i < n; i++)
	{
		out(paths[i]);
		cout << endl;
	}
	return ans;
}

signed main() {
	ios_base::sync_with_stdio(0);
	cin.tie(0);

	int t;
	cin >> t;
	for (int i = 0; i < t; ++i) {
		cout << solve() << endl;
	}

	return 0;
}