#include <iostream>
#include <fstream>

#include <set>
#include <vector>
#include <stack>

#include <cstring>
#include <cassert>
#include <cmath>

#include <algorithm>
#include <functional>

using namespace std;
//#define DEBUG
#ifdef  DEBUG
ifstream input("input.txt");
ofstream output("output.txt");
#define cin input
#define cout output
#endif //  DEBUG

typedef pair<int, int> pii;

int inf = 1000000000;
template<typename T> struct SegmentTree {
	int size, power, shift;
	T *base, *shared, def;
	function<T(T, T)> op;
	SegmentTree() {}
	SegmentTree(int p, T d, function<T(T, T)> oper, int fillv) {
		op = oper;
		power = p;
		def = d;
		size = (1 << (p + 1));
		shift = (1 << p) - 1;
		base = new T[size];
		fill_n(base, size, fillv);
		shared = base + shift;
	}

	void Update(int index, T val) {
		index += shift;
		base[index] = val;
		while (index) {
			index = (index - 1) / 2;
			base[index] = op(base[index * 2 + 1], base[index * 2 + 2]);
		}
	}
	
	T Query(int l, int r) {
		l += shift;
		r += shift;
		T ans = def;
		while (r > l) {
			if (!(l & 1)) {
				ans = op(base[l], ans);
				l++;
			}

			if (r & 1) {
				ans = op(base[r], ans);
				r--;
			}

			l = (l - 1) >> 1;
			r = (r - 1) >> 1;
		}

		if (l == r)ans = op(ans, base[l]);
		return ans;
	}
};

struct NearestMore {
	int *base;
	int size;
	NearestMore(int sz) {
		size = sz;
		base = new int[sz];
		fill_n(base, sz, 0);
	}
	void Update(int x, int val) {
		base[x] = val;
	}
	int Less(int ind, int minel) {
		for (int i = ind; i >= 0; i--) {
			if (base[i] >= minel)return i;
		}
		return -1;
	}

	int More(int ind, int minel) {
		for (int i = ind; i < size; i++) {
			if (base[i] >= minel)return i;
		}
		return -1;
	}
};

function<int(int, int)> minimum = (function<int(int, int)>)[](int a, int b) {return (a < b) ? a : b; };
int n, m, q;
SegmentTree<int> min_nr, min_mc;
NearestMore near_nr(1), near_mc(1);

set<pii> nodes(int r, int c, int kt) {
	set<pii> res;
	int val;

	if (near_mc.base[c] >= kt) {
		val = near_nr.More(r, kt);
		if (val != -1)res.insert({ val,c });

		val = near_nr.Less(r, kt);
		if (val != -1)res.insert({ val,c });
	}

	if (near_nr.base[r] >= kt) {
		val = near_mc.More(c, kt);
		if (val != -1)res.insert({ r, val });

		val = near_mc.Less(c, kt);
		if (val != -1)res.insert({ r, val });
	}

	return res;
}

int dist(pii p1, pii p2) {
	return (abs(p1.first - p2.first) + abs(p1.second - p2.second));
}

int main() {
	ios::sync_with_stdio(false);
	cin.tie(NULL);

	cin >> n >> m >> q;
	n += 2;
	m += 2;

	min_nr = SegmentTree<int>((int)ceil(log2(n + 3)) + 1, inf, minimum,0);
	min_mc = SegmentTree<int>((int)ceil(log2(m + 3)) + 1, inf, minimum,0);
	near_nr = NearestMore(n + 3);
    near_mc = NearestMore(m + 3);

	int t = 0;
	for(int i = 0; i< q; i++)
	{
		t++;
		int num;
		cin >> num;
		if (num == 1) {
			int r; cin >> r;
			min_nr.Update(r, t);
			near_nr.Update(r, t);
		}

		if (num == 2) {
			int c; cin >> c;
			min_mc.Update(c, t);
			near_mc.Update(c, t);
		}

		if (num == 3) {
			int c1, r1, c2, r2, k;
			cin >> r1 >> c1 >> r2 >> c2 >> k;
			k = t - k;
			
			//check manhattan
			if ((min_nr.Query(min(r1, r2), max(r1, r2)) >= k) || (min_mc.Query(min(c1, c2), max(c1, c2)) >= k)){
				
				cout << dist({r1,c1}, {r2,c2}) << "\n";
				continue;
			}

			set<pii> p1, p2;
			p1= nodes(r1, c1, k), p2 = nodes(r2, c2, k);
			int best = inf;
			for (auto v1 : p1) {
				for (auto v2 : p2) {
					int tec = dist({ r1,c1 }, v1) + dist(v1, v2) + dist(v2, { r2,c2 });
					if (tec <= best) best = tec;
				}
			}

			cout << ((best == inf) ? -1 : best ) << "\n";
		}
	}

	return 0;
}