#include <iostream>
#include <vector>
#include <algorithm>

using namespace std;

#define endl '\n'

struct node {
	int val;
	int l, r;
	node* left;
	node* right;
	node(int val, int l, int r, node* left = nullptr, node* right = nullptr) :
		val(val), l(l), r(r), left(left), right(right) {}
};

node* build(int l, int r) {
	if (r - l == 1) {
		return new node(0, l, r);
	}
	int m = (l + r) / 2;
	auto v = new node(0, l, r, build(l, m), build(m, r));
	return v;
}

const int INF = 1e6;

int get_min(node* v, int l, int r) {
	if (v->r <= l || v->l >= r) return INF;
	if (v->l >= l && v->r <= r) {
		return v->val;
	}
	return min(get_min(v->left, l, r), get_min(v->right, l, r));
}

void relax(node* v) {
	v->val = min(v->left->val, v->right->val);
	return;
}

void change(node* v, int l, int r, int x) {
	if (v->r <= l || v->l >= r) return;
	if (v->l >= l && v->r <= r) {
		v->val = x;
		return;
	}
	change(v->left, l, r, x);
	change(v->right, l, r, x);
	relax(v);
	return;
}

int s(int x1, int x2, int x) {
	return min(abs(x1 - x), abs(x2 - x));
}

int main() {
	ios_base::sync_with_stdio(false);
	cin.tie(0);

	int n, m, q;
	cin >> n >> m >> q;

	vector<int> str(n);
	vector<int> stb(m);
	node* rroot = build(0, n);
	node* broot = build(0, m);
	
	int t = 1;
	for (int i = 0; i < q; i++, t++) {
		int type;
		cin >> type;
		if (type == 1) {
			int ind;
			cin >> ind;
			str[--ind] = t;
			change(rroot, ind, ind + 1, t);
		}
		else if (type == 2) {
			int ind;
			cin >> ind;
			stb[--ind] = t;
			change(broot, ind, ind + 1, t);
		}
		else {
			int x1, y1, x2, y2, k;
			cin >> x1 >> y1 >> x2 >> y2 >> k;

			--x1; --y1; --x2; --y2;
			if ((str[x1] < t - k && stb[y1] < t - k) ||
				(str[x2] < t - k && stb[y2] < t - k)) {
				cout << -1 << endl;
				continue;
			}

			if ((str[x1] >= t - k && stb[y2] >= t - k) ||
				(stb[y1] >= t - k && str[x2] >= t - k)) {
				cout << abs(x1 - x2) + abs(y1 - y2) << endl;
				continue;
			}

			if (str[x1] >= t - k && str[x2] >= t - k) {
				int need = -1;
				for (int j = 0; j < m; j++) {
					if (stb[j] >= t - k && (need == -1 || s(y1, y2, j) < s(y1, y2, need))) {
						need = j;
					}
				}

				if (need == -1) {
					cout << -1 << endl;
					continue;

				}

				int ans = 2 * s(y1, y2, need);

				int x = get_min(rroot, min(x1, x2), max(x1, x2) + 1);
				if (x >= t - k) {
					ans = 0;
				}

				cout << abs(x1 - x2) + abs(y1 - y2) + ans << endl;
			}
			else {
				int need = -1;
				for (int j = 0; j < n; j++) {
					if (str[j] >= t - k && (need == -1 || s(x1, x2, j) < s(x1, x2, need))) {
						need = j;
					}
				}

				if (need == -1) {
					cout << -1 << endl;
					continue;

				}

				int ans = 2 * s(x1, x2, need);

				int x = get_min(broot, min(y1, y2), max(y1, y2) + 1);
				if (x >= t - k) {
					ans = 0;
				}

				cout << abs(x1 - x2) + abs(y1 - y2) + ans << endl;
			}
		}
	}
}