import java.io.PrintWriter;
import java.util.*;

class segment_tree {
    int max[], min[];
    int inf = (int) 1e9;

    segment_tree(int n) {
        max = new int[n * 4];
        min = new int[n * 4];
    }
    int get_min(int l, int r) {
        return get_min(0, 0, max.length / 4 - 1, l, r);
    }

    int get_min(int v, int l, int r, int left, int right) {
        if (l > right || r < left) return inf;
        if (l >= left && r <= right) return min[v];
        int m = (l + r) / 2;
        return Math.min(get_min(v * 2 + 1, l, m, left, right), get_min(v * 2 + 2, m + 1, r, left, right));
    }

    void update(int v, int l, int r, int id, int c) {
        if (l > id || r < id) return;
        if (l == id && r == id) {
            max[v] = c;
            min[v] = c;
            return;
        }
        int m = (l + r) / 2;
        update(v * 2 + 1, l, m, id, c);
        update(v * 2 + 2, m + 1, r, id, c);
        max[v] = Math.max(max[v * 2 + 1], max[v * 2 + 2]);
        min[v] = Math.min(min[v * 2 + 1], min[v * 2 + 2]);
    }

    void update(int id, int c) {
        update(0, 0, min.length / 4 - 1, id, c);
    }

    int pref_id(int v, int l, int r, int id, int t) {
        if (max[v] < t) return inf;
        if (r < id) return inf;
        if (l == r) return l;
        int m = (l + r) / 2;
        int ans = pref_id(v * 2 + 1, l, m, id, t);
        if (ans == inf) ans = pref_id(v * 2 + 2, m + 1, r, id, t);
        return ans;
    }

    int pref_id(int id, int t) {
        return pref_id(0, 0, min.length / 4 - 1, id, t);
    }

    int suf_id(int id, int t) {
        return suf_id(0, 0, min.length / 4 - 1, id, t);
    }

    int suf_id(int v, int l, int r, int id, int t) {
        if (max[v] < t) return -1;
        if (l > id) return -1;
        if (l == r) return l;
        int m = (l + r) / 2;
        int ans = suf_id(v * 2 + 2, m + 1, r, id, t);
        if (ans == -1) ans = suf_id(v * 2 + 1, l, m, id, t);
        return ans;
    }
}

public class Main {
    static int n, m;
    static segment_tree one, two;
    static int inf = (int) 1e9;

    static int third_type(int x1, int y1, int x2, int y2, int k, int t) {
        int ans = inf;
        int min = Math.abs(x1 - x2) + Math.abs(y1 - y2);
        boolean all_two = false;
        boolean all_one = false;
        boolean have_one = false;
        boolean have_two = false;

        if (t - one.get_min(x1, x1) <= k) all_two = true;
        if (t - two.get_min(y1, y1) <= k) all_one = true;
        if (t - one.get_min(x2, x2) <= k) have_one = true;
        if (t - two.get_min(y2, y2) <= k) have_two = true;

        if ((!have_one && !have_two) || (!all_two && !all_one)) {
            return -1;
        }

        boolean can1 = true;
        if (t - one.get_min(Math.min(x1, x2), Math.max(x1,x2)) > k) can1 = false;
        boolean can2 = true;
        if (t - two.get_min(Math.min(y1, y2), Math.max(y1,y2)) > k) can2 = false;

        if (can1 || can2) ans = min;

        if ((all_one && have_one) || (all_two && have_two)) ans = min;

        if (all_one && have_two) {
            int max_x = one.suf_id(x2, t - k);
            int min_x = one.pref_id(x2, t - k);
            int maybe1 = inf;
            if (max_x != -1) maybe1 = Math.abs(x1 - max_x) + Math.abs(y1 - y2) + Math.abs(max_x - x2);
            int maybe2 = inf;
            if (min_x != inf) maybe2 = Math.abs(min_x - x1) + Math.abs(y1 - y2) + Math.abs(min_x - x2);
            ans = Math.min(ans, Math.min(maybe1, maybe2));
        }

        if (all_two && have_one) {
            int max_y = two.suf_id(y2, t - k);
            int min_y = two.pref_id(y2, t - k);
            int maybe1 = inf;
            int maybe2 = inf;
            if (max_y != -1) maybe1 = Math.abs(y1 - max_y) + Math.abs(x1 - x2) + Math.abs(max_y - y2);
            if (min_y != inf) maybe2 = Math.abs(min_y - y1) + Math.abs(x1 - x2) + Math.abs(min_y - y2);
            ans = Math.min(ans, Math.min(maybe1, maybe2));
        }
        return ans == inf ? -1 : ans;
    }

    public static void main(String[] args) {
        Scanner sc = new Scanner(System.in);
        n = sc.nextInt();
        m = sc.nextInt();
        int q = sc.nextInt();
        one = new segment_tree(n);
        two = new segment_tree(m);
        for (int t = 1; t <= q; t++) {
            int type = sc.nextInt();
            if (type == 1) one.update(sc.nextInt() - 1, t);
            if (type == 2) two.update(sc.nextInt() - 1, t);
            if (type == 3) {
                int x1 = sc.nextInt() - 1;
                int y1 = sc.nextInt() - 1;
                int x2 = sc.nextInt() - 1;
                int y2 = sc.nextInt() - 1;
                int k = sc.nextInt();
                pw.println(third_type(x1, y1, x2, y2, k, t));
            }
        }
        pw.close();
    }

    static PrintWriter pw = new PrintWriter(System.out);
}
