﻿#include <iostream>
#include <algorithm>>
#include <iomanip>
#include <vector>
#include <cmath>
#include <string>
#include <queue>

using namespace std;

typedef long double ld;
typedef long long ll;
typedef unsigned short us;

#define all(x) x.begin(), x.end()

const int INF = 1e9;

struct node
{
	node *l;
	node *r;
	int mini;
	int maxi;
	int x;
	int prior;
	int subTreeSize;
	node() {}

	node(int _x)
	{
		l = r = nullptr;
		mini = maxi = x = _x;
		prior = rand() * rand();
		subTreeSize = 1;
	}
};

struct GotNode
{
	node *left;
	node *right;
	GotNode(node *l, node *r)
	{
		left = l;
		right = r;
	}
	GotNode() {};
};

int getSize(node *root)
{
	if (root == nullptr)
		return 0;
	return root->subTreeSize;
}

int getMaxi(node *root)
{
	if (root == nullptr)
		return -INF;
	return root->maxi;
}

int getMini(node *root)
{
	if (root == nullptr)
		return INF;
	return root->mini;
}

void update(node *root)
{
	if (root == nullptr)
		return;
	root->subTreeSize = getSize(root->l) + getSize(root->r) + 1;
	root->maxi = max({ getMaxi(root->l), getMaxi(root->r), root->x});
	root->mini = min({ getMini(root->l), getMini(root->r), root->x});
}

GotNode split(node *root, int key)
{
	if (root == nullptr)
	{
		return { nullptr, nullptr };
	}
	else if (getSize(root->l) + 1 >= key)
	{
		auto tmp = split(root->l, key);
		root->l = tmp.right;
		update(root);
		return { tmp.left, root };
	}
	else
	{
		auto tmp = split(root->r, key - getSize(root->l) - 1);
		root->r = tmp.left;
		update(root);
		return { root, tmp.right };
	}
}

node *merge(node *l, node *r)
{
	if (l == nullptr)
		return r;
	else if (r == nullptr)
		return l;
	else if (l->prior > r->prior)
	{
		l->r = merge(l->r, r);
		update(l);
		return l;
	}
	else
	{
		r->l = merge(l, r->l);
		update(r);
		return r;

	}
}

vector <int> horiz;
vector <int> vert;

bool canHor(int l, int r, int k, node *verti)
{
	if (l > r)
		swap(l, r);

	auto leftRight = split(verti, r + 1);
	auto leftMid = split(leftRight.left, l);
	bool answer = false;
	if (getMaxi(leftMid.right) <= k)
		answer = true;
	verti = merge(merge(leftMid.left, leftMid.right), leftRight.right);
	return answer;
	
}

bool canVert(int l, int r, int k, node *hori)
{
	if (l > r)
		swap(l, r);

	auto leftRight = split(hori, r + 1);
	auto leftMid = split(leftRight.left, l);
	bool answer = false;
	if (getMaxi(leftMid.right) <= k)
		answer = true;
	hori = merge(merge(leftMid.left, leftMid.right), leftRight.right);
	return answer;
}

struct qr
{
	int type;
	int line;
	int lx, ly, rx, ry, k;
	qr(int _type, int _line, int _lx, int _ly, int _rx, int _ry, int _k)
	{
		type = _type;
		line = _line;
		lx = _lx;
		rx = _rx;
		ly = _ly;
		ry = _ry;
		k = _k;
	}
	qr() {};
	void scan()
	{
		cin >> type;
		if (type == 1 || type == 2)
			cin >> line;
		else
			cin >> lx >> ly >> rx >> ry >> k;
	}
};

int get(node *root, int key)
{
	auto tmp = split(root, key);
	auto midRight = split(tmp.right, 2);
	int ans = midRight.left->x;
	root = merge(tmp.left, merge(midRight.left, midRight.right));
	return ans;
}

int indexToRight(node *root, int normal)
{
	if (root == nullptr)
		return -INF;
	if (getMini(root->r) <= normal)
		return indexToRight(root->r, normal) + getSize(root->l) + 1;
	if (root->x <= normal)
		return getSize(root->l) + 1;
	return indexToRight(root->l, normal);
}

int indexToLeft(node *root, int normal)
{
	if (root == nullptr)
		return INF;
	if (getMini(root->l) <= normal)
		return indexToLeft(root->l, normal);

	if (root->x <= normal)
		return getSize(root->l) + 1;

	return indexToLeft(root->r, normal) + getSize(root->l) + 1;
}

vector <int> solveSmart(int n, int m, int q, vector <qr> querys)
{
	horiz.resize(n + 1, 0);
	vert.resize(m + 1, 0);
	node *horTree = nullptr;
	node *vertTree = nullptr;

	for (int i = 1; i <= n; i++)
	{
		horTree = merge(horTree, new node(0));
	}

	for (int i = 1; i <= m; i++)
	{
		vertTree = merge(vertTree, new node(0));
	}

	vector <int> answer;
	for (int zz = 1; zz <= q; zz++)
	{
		int type = querys[zz].type;
		if (type == 1 || type == 2)
		{
			int line = querys[zz].line;

			if (type == 1) //Горизонтальные линии обновление 
			{
				auto leftRight = split(horTree, line);
				auto midRight = split(leftRight.right, 2);
				horTree = merge(merge(leftRight.left, new node(-zz)), midRight.right);
			}
			else //Вертикальные линии
			{
				auto leftRight = split(vertTree, line);
				auto midRight = split(leftRight.right, 2);
				vertTree = merge(merge(leftRight.left, new node(-zz)), midRight.right); //Вертикальные линии обновление
			}
		}
		else
		{
			int lx, ly, rx, ry, k;
			lx = querys[zz].lx;
			ly = querys[zz].ly;
			rx = querys[zz].rx;
			ry = querys[zz].ry;
			k = querys[zz].k;

			k -= zz;

			int minAns = INF;
			if (canHor(ly, ry, k, vertTree) || canVert(lx, rx, k, horTree)) //онли вертикальные и горизонтальные 
				minAns = abs(rx - lx) + abs(ry - ly);

			if (get(horTree, lx) <= k && get(vertTree, ry) <= k || get(vertTree, ly) <= k && get(horTree, rx) <= k)
				minAns = min(minAns, abs(rx - lx) + abs(ry - ly));

			if (lx > rx)
				swap(lx, rx);
			if (ly > ry)
				swap(ly, ry);
			//Обновление через две горизонтальных 

			if (get(horTree, lx) <= k && get(horTree, rx) <= k) //По двум горизонтальным и вертикальной
			{
				auto tmp = split(vertTree, ry + 1);
				auto leftMid = split(tmp.left, ly);

				node *right = tmp.right;
				node *left = leftMid.left;
				node *mid = leftMid.right;

				if (getMini(mid) <= k)
				{
					minAns = min(minAns, ry - ly + rx - lx);
				}

				int toRight = indexToRight(left, k);
				int toLeft = indexToLeft(right, k) + ry;
				
				minAns = min(minAns, toLeft - ry + toLeft - ly + rx - lx);
				minAns = min(minAns, ly - toRight + ry - toRight + rx - lx);

				vertTree = merge(merge(left, mid), right);
			}

			if (get(vertTree, ly) <= k && get(vertTree, ry) <= k) //По двум вертикальным и горизонтальной
			{
				auto tmp = split(horTree, rx + 1);
				auto leftMid = split(tmp.left, lx);

				node *right = tmp.right;
				node *left = leftMid.left;
				node *mid = leftMid.right;

				if (getMini(mid) <= k)
				{
					minAns = min(minAns, ry - ly + rx - lx);
				}

				int toRight = indexToRight(left, k);
				int toLeft = indexToLeft(right, k) + rx;

				minAns = min(minAns, toLeft - rx + toLeft - lx + ry - ly);
				minAns = min(minAns, lx - toRight + rx - toRight + ry - ly);

				horTree = merge(merge(left, mid), right);
			}
			if (minAns == INF)
			{
				answer.push_back(-1);
			}
			else
			{
				answer.push_back(minAns);
			}
		}

	}
	return answer;
}


signed main()
{
#ifdef _DEBUG
	freopen("input.txt", "r", stdin);
	freopen("output.txt", "w", stdout);
#endif
	ios_base::sync_with_stdio(0);
	cin.tie(0);
	int q, n, m;
	cin >> n >> m >> q;
	vector <qr> querys(q + 1);
	for (int i = 1; i <= q; i++)
		querys[i].scan();

	vector <int> answer2 = solveSmart(n, m, q, querys);

	for (int i = 0; i < answer2.size(); i++)
	{
		cout << answer2[i] << endl;
	}
	
	
	return 0;
}