#define _CRT_SECURE_NO_WARNINGS

#include <iostream>
#include <vector>
#include <algorithm>
#include <cmath>
#include <set>

using namespace std;

#define endl "\n"
#define mp make_pair

const int BLOCK = 550;
const int N = (int)3e5 + 7;
const int INF = (int)1e9 + 7;

///////////////////////////////////////////////////

struct Query {
	int l, r, ind;
	Query() {};
	Query(int l, int r, int ind) :
		l(l), r(r), ind(ind) {};
};

bool operator < (const Query &a, const Query &b) {
	if (a.l / BLOCK == b.l / BLOCK && a.r == b.r)
		return a.ind < b.ind;
	if (a.l / BLOCK == b.l / BLOCK)
		return a.r < b.r;
	return a.l / BLOCK < b.l / BLOCK;
}

pair<int, int> p1[N], p2[N];
int n, m, t, ans[N];
Query q[N];

vector <int> pev;
vector <pair<int, int>> ev;

int row[N], col[N];
int br[N], bc[N];

///////////////////////////////////////////////////

int get_dist(int a1, int b1, int a2, int b2) {
	return abs(a1 - a2) + abs(b1 - b2);
}

void upd_line(int x, int val, int *a, int *b) {
	a[x] += val;
	if (a[x] == 0 && val == -1)
		b[x / BLOCK]--;
	if (a[x] == 1 && val == 1)
		b[x / BLOCK]++;
}

int get_sum(int l, int r, int *a, int *b) {
	int res = 0, L = l / BLOCK, R = r / BLOCK;
	for (int i = L + 1; i < R; i++)
		res += b[i];
	for (int i = l; i <= r && i < (L + 1) * BLOCK; i++)
		res += (a[i] > 0 ? 1 : 0);
	if (L != R) {
		for (int i = R * BLOCK; i <= r; i++)
			res += (a[i] > 0 ? 1 : 0);
	}
	return res;
}

int get_left(int x, int *a, int *b) {
	int bl = x / BLOCK;
	for (int i = x; i >= bl * BLOCK; i--) {
		if (a[i] > 0)
			return i;
	}
	for (int i = bl - 1; i >= 0; i--) {
		if (b[i] > 0) {
			for (int j = (i + 1) * BLOCK - 1;; j--) {
				if (a[j] > 0)
					return j;
			}
		}
	}
	return -1;
}

int get_right(int x, int *a, int *b) {
	int bl = x / BLOCK;
	for (int i = x; i < (bl + 1) * BLOCK; i++) {
		if (a[i] > 0)
			return i;
	}
	for (int i = bl + 1; i * BLOCK <= max(n, m); i++) {
		if (b[i] > 0) {
			for (int j = i * BLOCK;; j++) {
				if (a[j] > 0)
					return j;
			}
		}
	}
	return INF;
}

int get_ans(int ind) {
	int a1 = p1[ind].first, b1 = p1[ind].second, a2 = p2[ind].first, b2 = p2[ind].second;
	if (row[a1] == 0 && col[b1] == 0)
		return -1;
	if (row[a2] == 0 && col[b2] == 0)
		return -1;
	int dst = get_dist(a1, b1, a2, b2);
	if (row[a1] > 0 && col[b2] > 0)
		return dst;
	if (col[b1] > 0 && row[a2] > 0)
		return dst;
	if (row[a1] > 0 && row[a2] > 0) {
		int l = min(a1, a2), r = max(a1, a2);
		int sum = get_sum(l, r, row, br);
		if (sum == r - l + 1)
			return dst;
		l = min(b1, b2), r = max(b1, b2);
		sum = get_sum(l, r, col, bc);
		if (sum > 0)
			return dst;
		int L = get_left(l, col, bc), R = get_right(r, col, bc);
		if (L == -1 && R == INF)
			return -1;
		int r1 = dst + 2 * (l - L), r2 = dst + 2 * (R - r);
		if (L == -1)
			return r2;
		if (R == INF)
			return r1;
		return min(r1, r2);
	}
	else {
		int l = min(b1, b2), r = max(b1, b2);
		int sum = get_sum(l, r, col, bc);
		if (sum == r - l + 1)
			return dst;
		l = min(a1, a2), r = max(a1, a2);
		sum = get_sum(l, r, row, br);
		if (sum > 0)
			return dst;
		int L = get_left(l, row, br), R = get_right(r, row, br);
		if (L == -1 && R == INF)
			return -1;
		int r1 = dst + 2 * (l - L), r2 = dst + 2 * (R - r);
		if (L == -1)
			return r2;
		if (R == INF)
			return r1;
		return min(r1, r2);
	}
}

int main() {
#ifdef _DEBUG
	freopen("input.txt", "r", stdin);
	freopen("output.txt", "w", stdout);
	cin.tie(0), cout.tie(0);
	ios_base::sync_with_stdio(0);
#else
	cin.tie(0), cout.tie(0);
	ios_base::sync_with_stdio(0);
#endif
	cin >> n >> m >> t;
	int cntq = 0;
	for (int i = 1; i <= t; i++) {
		int ct; cin >> ct;
		if (ct != 3) {
			int x; cin >> x;
			pev.push_back(i);
			ev.push_back(mp(ct, x));
		}
		else {
			int k;
			cin >> p1[i].first >> p1[i].second >> p2[i].first >> p2[i].second >> k;
			if (pev.size() == 0) {
				if (k >= i)
					ans[i] = get_dist(p1[i].first, p1[i].second, p2[i].first, p2[i].second);
				else
					ans[i] = -1;
			}
			else {
				int l = lower_bound(pev.begin(), pev.end(), i - k) - pev.begin(), r = (int)pev.size() - 1;
				if (l == (int)pev.size())
					ans[i] = -1;
				else
					q[cntq++] = Query(l, r, i);
			}
		}
	}
	sort(q, q + cntq);
	int l = 0, r = -1;
	for (int i = 0; i < cntq; i++) {
		while (r < q[i].r) {
			r++;
			if (ev[r].first == 1)
				upd_line(ev[r].second, 1, row, br);
			else
				upd_line(ev[r].second, 1, col, bc);
		}
		while (r > q[i].r) {
			if (ev[r].first == 1)
				upd_line(ev[r].second, -1, row, br);
			else 
				upd_line(ev[r].second, -1, col, bc);
			r--;
		}
		while (l < q[i].l) {
			if (ev[l].first == 1)
				upd_line(ev[l].second, -1, row, br);
			else
				upd_line(ev[l].second, -1, col, bc);
			l++;
		}
		while (l > q[i].l) {
			l--;
			if (ev[l].first == 1)
				upd_line(ev[l].second, 1, row, br);
			else
				upd_line(ev[l].second, 1, col, bc);
		}
		ans[q[i].ind] = get_ans(q[i].ind);
	}
	for (int i = 0; i <= t; i++) {
		if (ans[i] == 0)
			continue;
		cout << ans[i] << endl;
	}
}