#define _CRT_SECURE_NO_WARNINGS

#include <iostream>
#include <algorithm>
#include <set>
#include <map>
#include <vector>
#include <string>
#include <cmath>
#include <time.h>

using namespace std;

#pragma GCC optimize("Ofast")
#pragma GCC optimize("unroll-loops")
#pragma GCC optimize("no-stack-protector")
#pragma GCC optimize("fast-math")

typedef long long ll;
typedef long double ld;

#define int long long

const int inf = 1e9 + 7;
const int eps = 1e-9;

const int k = sqrt(1e6);

struct Block
{
	int minv;
	int maxv;

	Block(int mi = inf, int ma = -1) : maxv(ma), minv(mi) {}

	void upd(int mi, int ma)
	{
		minv = min(minv, mi);
		maxv = max(maxv, ma);
	}
};

int maxi(int r, int x, vector<Block>& bl, vector<int>& arr)
{
	while (r >= 0 && r % k != k - 1)
	{
		if (arr[r] >= x)
			return r;
		r--;
	}
	while (r > 0)
	{
		if (bl[r / k].maxv >= x)
		{
			while (arr[r] < x)
			{
				r--;
			}
			return r;
		}
		r -= k;
	}
	return -1;
}

int mini(int l, int x, vector<Block>& bl, vector<int>& arr)
{
	while (l < arr.size() && l % k != 0)
	{
		if (arr[l] >= x)
			return l;
		l++;
	}
	while (l < arr.size())
	{
		if (bl[l / k].maxv >= x)
		{
			while (arr[l] < x)
			{
				l++;
			}
			return l;
		}
		l += k;
	}
	return -1;
}

Block extr(int l, int r, vector<Block>& bl, vector<int>& arr)
{
	Block ans;

	while (l <= r && l % k != 0)
	{
		ans.upd(arr[l], arr[l]);
		l++;
	}

	while (l <= r && r % k != k - 1)
	{
		ans.upd(arr[r], arr[r]);
		r--;
	}

	while (l <= r)
	{
		ans.upd(bl[l / k].minv, bl[l / k].maxv);
		l += k;
	}

	return ans;
}

void setval(int v, int i, vector<Block>& bl, vector<int>& arr)
{
	arr[i] = v;
	int bli = i / k;
	int l = bli * k;
	int r = min((int)arr.size(), l + k);

	bl[bli] = Block();
	for (int i = l; i < r; i++)
	{
		bl[bli].upd(arr[i], arr[i]);
	}
}

int getans(int x1, int y1, int x2, int y2, vector<int>& x, vector<Block>& xbl, vector<int>& y, vector<Block>& ybl, int minv)
{
	int ans = abs(x1 - x2) + abs(y1 - y2);

	int l = y1;
	int r = y2;
	if (r < l)
		swap(l, r);

	if (abs(x1 - x2) > 1)
	{
		Block mid = extr(l, r, ybl, y);
		if (mid.maxv >= minv)
			goto out;

		int l1 = x1, r1 = x2;
		if (r1 < l1)
			swap(l1, r1);
		Block gap = extr(l1 + 1, r1 - 1, xbl, x);
		if (gap.minv >= minv)
			goto out;

		int add = inf;

		int minind = mini(r + 1, minv, ybl, y);
		if (minind != -1)
			add = min(add, minind - r);

		int maxind = maxi(l - 1, minv, ybl, y);
		if (maxind != -1)
			add = min(add, l - maxind);

		if (add == inf)
			ans = -1;
		else
			ans += 2 * add;
	}
out:
	return ans;
}

signed main()
{
#ifdef LOCAL
	freopen("input.txt", "r", stdin);
	freopen("output.txt", "w", stdout);
#endif

	ios_base::sync_with_stdio(false);
	cin.tie(NULL);

	int n, m, q;
	cin >> n >> m >> q;

	vector<int> vert(n, 0);
	vector<int> hor(m, 0);
	int nsz = ceil((ld)n / k);
	int msz = ceil((ld)m / k);
	vector<Block> vbl(nsz, 0);
	vector<Block> hbl(msz, 0);

	for (int t = 1; t <= q; t++)
	{
		int type;
		cin >> type;

		if (type == 1)
		{
			int x;
			cin >> x;
			x--;

			setval(t, x, vbl, vert);
		}
		else if (type == 2)
		{
			int y;
			cin >> y;
			y--;

			setval(t, y, hbl, hor);
		}
		else
		{
			int x1, y1, x2, y2, k;
			cin >> x1 >> y1 >> x2 >> y2 >> k;
			x1--; y1--; x2--; y2--;

			int minv = t - k;

			if (vert[x1] >= minv && hor[y2] >= minv)
			{
				cout << abs(x1 - x2) + abs(y1 - y2) << '\n';
			}
			else if (vert[x2] >= minv && hor[y1] >= minv)
			{
				cout << abs(x1 - x2) + abs(y1 - y2) << '\n';
			}
			else if (vert[x1] >= minv && vert[x2] >= minv)
			{
				//vert
				cout << getans(x1, y1, x2, y2, vert, vbl, hor, hbl, minv) << '\n';
			}
			else if (hor[y1] >= minv && hor[y2] >= minv)
			{
				//hor
				cout << getans(y1, x1, y2, x2, hor, hbl, vert, vbl, minv) << '\n';
			}
			else
			{
				cout << -1 << '\n';
			}

		}
	}

	//system("pause");
	return 0;
}