#include <iostream>
#include <cmath>
#include <cstdlib>

using namespace std;

typedef long long ll;
typedef unsigned long long ull;
typedef long double ld;

const int N = 1e6 + 10, INF = 1e9 + 7;

struct node{
    int mx, mn;
    node *l, *r;
    node(int v) {
        mn = mx = v;
        l = r = 0;
    }
    node(node *a, node *b) {
        l = a;
        r = b;
        mx = max(l->mx, r->mx);
        mn = min(l->mn, r->mn);
    }
} *tx, *ty;

int n, m, q, x[N], y[N];

node* build(int *a, int tl, int tr) {
    if (tl == tr)
        return new node(a[tl]);
    int tm = (tl + tr) / 2;
    return new node(build(a, tl, tm), build(a, tm + 1, tr));
}

void update(node *&t, int tl, int tr, int &p, int &v) {
    if (tl == tr)
        t->mx = t->mn = v;
    else {
        int tm = (tl + tr) / 2;
        if (p <= tm)
            update(t->l, tl, tm, p, v);
        else
            update(t->r, tm + 1, tr, p, v);
        t->mx = max(t->l->mx, t->r->mx);
        t->mn = min(t->l->mn, t->r->mn);
    }
}

int get_max(node *t, int tl, int tr, int l, int r) {
    if (l > r)
        return -INF;
    if (l == tl && r == tr)
        return t->mx;
    int tm = (tl + tr) / 2;
    return max(get_max(t->l, tl, tm, l, min(tm, r)), get_max(t->r, tm + 1, tr, max(tm + 1, l), r));
}

int get_min(node *t, int tl, int tr, int l, int r) {
    if (l > r)
        return INF;
    if (l == tl && r == tr)
        return t->mn;
    int tm = (tl + tr) / 2;
    return min(get_min(t->l, tl, tm, l, min(tm, r)), get_min(t->r, tm + 1, tr, max(tm + 1, l), r));
}

int main() {
    ios::sync_with_stdio(0);
    cin.tie(0);
    cout.tie(0);

    cin >> n >> m >> q;
    tx = build(x, 0, n - 1);
    ty = build(y, 0, m - 1);
    for (int i = 1; i <= q; i++) {
        int t;
        cin >> t;
        if (t == 1) {
            int r;
            cin >> r;
            r--;
            x[r] = -i;
            update(tx, 0, n - 1, r, x[r]);
        }
        else if (t == 2) {
            int c;
            cin >> c;
            c--;
            y[c] = -i;
            update(ty, 0, m - 1, c, y[c]);
        }
        else {
            int r1, c1, r2, c2, k;
            cin >> r1 >> c1 >> r2 >> c2 >> k;
            r1--;
            r2--;
            c1--;
            c2--;
            if (r1 > r2 || r1 == r2 && c1 > c2) {
                swap(r1, r2);
                swap(c1, c2);
            }
            if (r1 == r2 && x[r1] + i <= k) {
                cout << abs(c2 - c1) << '\n';
                continue;
            }
            if (c1 == c2 && y[c1] + i <= k) {
                cout << abs(r2 - r1) << '\n';
                continue;
            }
            if (min(x[r1], y[c1]) + i > k || min(x[r2], y[c2]) + i > k) {
                cout << -1 << '\n';
                continue;
            }
            if (x[r1] + i <= k && x[r2] + i > k) {
                cout << abs(r2 - r1) + abs(c2 - c1) << '\n';
                continue;
            }
            if (x[r2] + i <= k && x[r1] + i > k) {
                cout << abs(r2 - r1) + abs(c2 - c1) << '\n';
                continue;
            }
            if (y[c1] + i <= k && y[c2] + i > k) {
                cout << abs(r2 - r1) + abs(c2 - c1) << '\n';
                continue;
            }
            if (y[c2] + i <= k && y[c1] + i > k) {
                cout << abs(r2 - r1) + abs(c2 - c1) << '\n';
                continue;
            }
            if (get_max(tx, 0, n - 1, r1, r2) + i <= k || get_max(ty, 0, m - 1, min(c1, c2), max(c1, c2)) + i <= k) {
                cout << abs(r2 - r1) + abs(c2 - c1) << '\n';
                continue;
            }
            int ans = INF;
            if (x[r1] + i <= k && x[r2] + i <= k) {
                if (get_min(ty, 0, m - 1, min(c1, c2), max(c1, c2)) + i <= k) {
                    cout << abs(r2 - r1) + abs(c2 - c1) << '\n';
                    continue;
                }
                if (get_min(ty, 0, m - 1, 0, min(c1, c2) - 1) + i <= k || get_min(ty, 0, m - 1, max(c1, c2) + 1, m - 1) + i <= k) {
                    int p = INF, a = min(c1, c2), b = max(c1, c2);
                    for (int j = 1; (a - j >= 0 || b + j < m) && j <= 100; j++)
                        if (a - j >= 0 && y[a - j] + i <= k || b + j < m && y[b + j] + i <= k) {
                            p = j;
                            break;
                        }
                    ans = min(ans, abs(r2 - r1) + abs(c2 - c1) + 2 * p);
                }
            }
            if (y[c1] + i <= k && y[c2] + i <= k) {
                if (get_min(tx, 0, n - 1, r1, r2) + i <= k) {
                    cout << abs(r2 - r1) + abs(c2 - c1) << '\n';
                    continue;
                }
                if (get_min(tx, 0, n - 1, 0, r1 - 1) + i <= k || get_min(tx, 0, n - 1, r2 + 1, n - 1) + i <= k) {
                    int p = INF;
                    for (int j = 1; (r1 - j >= 0 || r2 + j < n) && j <= 100; j++)
                        if (r1 - j >= 0 && x[r1 - j] + i <= k || r2 + j < n && x[r2 + j] + i <= k) {
                            p = j;
                            break;
                        }
                    ans = min(ans, abs(r2 - r1) + abs(c2 - c1) + 2 * p);
                }
            }
            cout << (ans == INF ? -1 : ans) << '\n';
        }
    }

    return 0;
}

/*
4 3 9
3 2 1 3 3 1
3 2 1 3 3 1
2 1
2 3
1 1
3 2 1 3 3 3
3 2 1 3 3 3
2 2
3 2 1 3 3 6
*/
