#include <algorithm>
#include <iostream>
#include <vector>

using ll = long long;

const int INF = 1e9;

struct SegmentTree {
    int size;
    std::vector<int> max;
    std::vector<int> min;

    void create(int n) {
        size = n;
        int t = 1;
        while (t <= size) {
            t <<= 1;
        }
        t <<= 1;
        max.resize(t);
        min.resize(t);
        //_build(1, 0, size - 1, a);
    }

//    void _build(int v, int curL, int curR, const std::vector<int> &a) {
//        if (curL == curR) {
//            min[v] = a[curL];
//            max[v] = a[curR];
//            return;
//        }
//        int mid = (curL + curR) >> 1;
//        _build(2 * v, curL, mid, a);
//        _build(2 * v + 1, mid + 1, curR, a);
//        _update(v);
//    }

    void update(int v, int curL, int curR, int ind, int val) {
        if (curL == curR) {
            min[v] = val;
            max[v] = val;
            return;
        }
        int mid = (curL + curR) >> 1;
        if (ind <= mid) {
            update(2 * v, curL, mid, ind, val);
        } else {
            update(2 * v + 1, mid + 1, curR, ind, val);
        }
        min[v] = std::min(min[2 * v], min[2 * v + 1]);
        max[v] = std::max(max[2 * v], max[2 * v + 1]);
    }

    void update(int ind, int val) {
        update(1, 0, size - 1, ind, val);
    }

    int get_min(int v, int curL, int curR, int l, int r) {
        if (curL > r || curR < l) {
            return INF;
        }
        if (curL >= l && curR <= r) {
            return min[v];
        }
        int mid = (curL + curR) >> 1;
        return std::min(get_min(2 * v, curL, mid, l, r), get_min(2 * v + 1, mid + 1, curR, l, r));
    }

    int get_min(int l, int r) {
        return get_min(1, 0, size - 1, l, r);
    }

    int get_max(int v, int curL, int curR, int l, int r) {
        if (curL > r || curR < l) {
            return -INF;
        }
        if (curL >= l && curR <= r) {
            return max[v];
        }
        int mid = (curL + curR) >> 1;
        return std::max(get_max(2 * v, curL, mid, l, r), get_max(2 * v + 1, mid + 1, curR, l, r));
    }

    int get_max(int l, int r) {
        return get_max(1, 0, size - 1, l, r);
    }
};

int n, m, q;

std::vector<int> line, column;
SegmentTree lt, ct;

int dist(int i1, int j1, int i2, int j2) {
    return std::abs(i1 - i2) + std::abs(j1 - j2);
}

int maxColumn(int l, int r) {
    if (l > r) {
        return maxColumn(r, l);
    }
    return ct.get_max(l, r);
}

int maxLine(int l, int r) {
    if (l > r) {
        return maxLine(r, l);
    }
    return lt.get_max(l, r);
}

int minColumn(int l, int r) {
    if (l > r) {
        return minColumn(r, l);
    }
    return ct.get_min(l, r);
}

int minLine(int l, int r) {
    if (l > r) {
        return minLine(r, l);
    }
    return lt.get_min(l, r);
}

int columnLeft(int ind, int k) {
    for (int i = ind - 1; i >= 0; --i) {
        if (column[i] >= k) {
            return i;
        }
    }
    return -1;
}

int lineLeft(int ind, int k) {
    for (int i = ind - 1; i >= 0; --i) {
        if (line[i] >= k) {
            return i;
        }
    }
    return -1;
}

int columnRight(int ind, int k) {
    for (int i = ind + 1; i < m; ++i) {
        if (column[i] >= k) {
            return i;
        }
    }
    return -1;
}

int lineRight(int ind, int k) {
    for (int i = ind + 1; i < n; ++i) {
        if (line[i] >= k) {
            return i;
        }
    }
    return -1;
}

int go(int i1, int j1, int i2, int j2, int k) {
    int st1 = 0, st2 = 0;
    if (line[i1] >= k) {
        st1 ^= 1;
    }
    if (line[i2] >= k) {
        st2 ^= 1;
    }
    if (column[j1] >= k) {
        st1 ^= 2;
    }
    if (column[j2] >= k) {
        st2 ^= 2;
    }
    if (st1 == 0 || st2 == 0) {
        return -1;
    }
    if ((st1 | st2) == 3) {
        return dist(i1, j1, i2, j2);
    }
    if (st1 == 1) {
        if (maxColumn(j1, j2) >= k || minLine(i1, i2) >= k) {
            return dist(i1, j1, i2, j2);
        }
        int l = j1, r = j2;
//        if (l > r) {
//            std::swap(l, r);
//        }
//        int pl = columnLeft(l, k), pr = columnRight(r, k);
//        if (pl == -1 && pr == -1) {
//            return -1;
//        }
//        if (pl == -1) {
//            return dist(i1, j1, i2, j2) + 2 * (pr - r);
//        }
//        if (pr == -1) {
//            return dist(i1, j1, i2, j2) + 2 * (l - pl);
//        }
//        return dist(i1, j1, i2, j2) + 2 * (std::min(pr - r, l - pl));
    } else {
        if (maxLine(i1, i2) >= k || minColumn(j1, j2) >= k) {
            return dist(i1, j1, i2, j2);
        }
        int l = i1, r = i2;
//        if (l > r) {
//            std::swap(l, r);
//        }
//        int pl = lineLeft(l, k), pr = lineRight(r, k);
//        if (pl == -1 && pr == -1) {
//            return -1;
//        }
//        if (pl == -1) {
//            return dist(i1, j1, i2, j2) + 2 * (pr - r);
//        }
//        if (pr == -1) {
//            return dist(i1, j1, i2, j2) + 2 * (l - pl);
//        }
//        return dist(i1, j1, i2, j2) + 2 * (std::min(pr - r, l - pl));
    }
    return -1;
}

int main() {
    std::ios::sync_with_stdio(0);
    std::cin.tie(0);
    std::cout.tie(0);
    std::cout.precision(20);
//    freopen("input", "r", stdin);
    std::cin >> n >> m >> q;
    line.resize(n);
    column.resize(m);
    lt.create(n);
    ct.create(m);
    for (int time = 1; time <= q; ++time) {
        int t;
        std::cin >> t;
        if (t == 1) {
            int ind;
            std::cin >> ind;
            --ind;
            line[ind] = time;
            lt.update(ind, time);
            continue;
        }
        if (t == 2) {
            int ind;
            std::cin >> ind;
            --ind;
            column[ind] = time;
            ct.update(ind, time);
            continue;
        }
        int i1, j1, i2, j2, k;
        std::cin >> i1 >> j1 >> i2 >> j2 >> k;
        --i1;
        --j1;
        --i2;
        --j2;
        k = time - k;
        std::cout << go(i1, j1, i2, j2, k) << '\n';
    }
    return 0;
}