#include <assert.h>
#include <algorithm>
#include <iostream>
#include <vector>

using ll = long long;

struct Treap {
    Treap *left;
    Treap *right;
    ll x;
    int y;
    ll sum;
    Treap(ll x = 0ll): left(nullptr), right(nullptr), x(x), y((rand() << 16) + rand()), sum(x) {}
};

using ptreap = Treap*;

ll getSum(ptreap t) {
    return t ? t->sum : 0ll;
}

void update(ptreap t) {
    if (t) {
        t->sum = getSum(t->left) + getSum(t->right) + t->x;
    }
}

ptreap merge(ptreap a, ptreap b) {
    if (!a) return b;
    if (!b) return a;
    if (a->y > b->y) {
        a->right = merge(a->right, b);
        update(a);
        return a;
    } else {
        b->left = merge(a, b->left);
        update(b);
        return b;
    }
}

void split(ptreap t, ll key, ptreap &l, ptreap &r) {
    if (!t) {
        l = r = nullptr;
        return;
    }
    if (key >= t->x) {
        split(t->right, key, t->right, r);
        l = t;
    } else {
        split(t->left, key, l, t->left);
        r = t;
    }
    update(l);
    update(r);
}

ptreap trp = nullptr;

void pushAdd(ll x) {
    ptreap l, r;
    split(trp, x, l, r);
    l = merge(l, new Treap(x));
    trp = merge(l, r);
}

bool isCan(ll lg, ll rg) {
    ptreap l, r;
    split(trp, rg, l, r);
    bool ans = (getSum(l) >= lg);
    trp = merge(l, r);
    return ans;
}

struct Query {
    int ind;
    int v;
    ll l;
    ll r;
    char ans;

    bool in(ll x) {
        return x >= l && x <= r;
    }
};

int n, m, q;
ll p;
std::vector<std::vector<std::pair<int, ll>>> graph;
std::vector<Query> qs;

void solve() {
    graph.clear();
    qs.clear();
    std::cin >> n >> m >> q >> p;
    graph.resize(n);
    qs.resize(q);
    assert(m == 2 * (n - 1));
    for (int i = 0; i < m; ++i) {
        int u, v;
        ll d;
        std::cin >> u >> v >> d;
        --u;
        --v;
        graph[u].push_back({v, d});
    }
    for (int i = 0; i < q; ++i) {
        bool fl = 1;
        int f;
        ll l;
        std::cin >> f >> l;
        --f;
        qs[i].v = f;
        ll r = (l * p) / (p - 1);
        qs[i].ind = i;
        qs[i].l = l;
        qs[i].r = r;
    }
    std::sort(qs.begin(), qs.end(), [&] (const Query &a, const Query &b) {
        return a.v < b.v;
    });
    for (int i = 0; i < n - 1; ++i) {
        if (graph[i][0].second > graph[i][1].second) {
            std::swap(graph[i][0], graph[i][1]);
        }
    }
    ll cur_sum = 0;
    int indq = 0;
    for (int i = 0; i < n - 1; ++i) {
        while (indq < qs.size() && qs[indq].v == i) {
            if (qs[indq].in(cur_sum)) {
                qs[indq].ans = '1';
                ++indq;
                continue;
            }
            if (qs[indq].r < cur_sum) {
                qs[indq].ans = '0';
                ++indq;
                continue;
            }
            ll l1 = std::max(0ll, qs[indq].l - cur_sum), r1 = std::max(0ll, qs[indq].r - cur_sum);
            if (isCan(l1, r1)) {
                qs[indq].ans = '1';
            } else {
                qs[indq].ans = '0';
            }
            ++indq;
        }
        cur_sum += graph[i][0].second;
        pushAdd(graph[i][1].second - graph[i][0].second);
    }
    while (indq < qs.size() && qs[indq].v == n - 1) {
        if (qs[indq].in(cur_sum)) {
            qs[indq].ans = '1';
            ++indq;
            continue;
        }
        if (qs[indq].r < cur_sum) {
            qs[indq].ans = '0';
            ++indq;
            continue;
        }
        ll l1 = std::max(0ll, qs[indq].l - cur_sum), r1 = std::max(0ll, qs[indq].r - cur_sum);
        if (isCan(l1, r1)) {
            qs[indq].ans = '1';
        } else {
            qs[indq].ans = '0';
        }
        ++indq;
    }
    std::sort(qs.begin(), qs.end(), [&] (const Query &a, const Query &b) {
        return a.ind < b.ind;
    });
    for (auto i : qs) {
        std::cout << i.ans;
    }
    std::cout << '\n';
}

int main() {
    std::ios::sync_with_stdio(0);
    std::cin.tie(0);
    std::cout.tie(0);
    std::cout.precision(20);
//    freopen("input", "r", stdin);
    int cntTests = 0;
    std::cin >> cntTests;
    while (cntTests--) {
        solve();
    }
    return 0;
}
