#define _CRT_SECURE_NO_WARNINGS

#include <iostream>
#include <set>
#include <map>
#include <algorithm>
#include <stack>
#include <queue>
#include <iomanip>
#include <bitset>
#include <cmath>

using namespace std;

typedef long long ll;
typedef long double ld;

const int INF = 1e9;
const ll INF_LL = 1e18;

//#pragma GCC optimize("Ofast,unroll-loops")
//#pragma GCC target("abm,mmx,avx,avx2,sse,sse2,ssse3,sse4,tune=native")

struct Seg {
	int l, r;
	Seg(int l_ = 0, int r_ = 0) {
		l = l_;
		r = r_;
	}

	Seg isect(Seg b) {
		return Seg(max(l, b.l), min(r, b.r));
	}

	bool in(int x) {
		return (x < r) && (x >= l);
	}
};

struct Node {
	int ma, mi;

	Node(int ma_ = 0, int mi_ = 0) {
		ma = ma_;
		mi = mi_;
	}
};

struct Tree {
	int n;
	int d;
	vector<int> val;
	
	Tree(int n_ = 0) {
		n = n_;
		//val = new int[4 * n];
		//fill(val, val + n, 0);
		val.assign(n, 0);
		d = 0;
	}

	void mod(int x) {
		val[x] = -d;
	}

	int get_ma(int l, int r) {
		int ans = -INF;
		for (int i = l; i < r; i++)
			ans = max(ans, val[i]);
		if (ans == -INF)
			return ans;
		return ans + d;
	}

	int get_mi(int l, int r) {
		int ans = INF;
		for (int i = l; i < r; i++)
			ans = min(ans, val[i]);
		if (ans == INF)
			return ans;
		return ans + d;
	}

	int find_first_gr(int x, int k) {
		for (int i = x; i < n; i++) {
			if (val[i] + d <= k)
				return i;
		}
		return n;
	}

	int find_first_less(int x, int k) {
		for (int i = x; i >= 0; i--) {
			if (val[i] + d <= k)
				return i;
		}
		return -1;
	}

	Seg find_range(int x, int k) {
		Seg ans;
		int pos = x;
		while (pos < n && val[pos] + d <= k)
			pos++;
		ans.r = pos;
		pos = x;
		while (pos > 0 && val[pos - 1] + d <= k)
			pos--;
		ans.l = pos;
		
		return ans;
	}
};

int get_ver(int r1, int c1, int r2, int c2, int k, Tree &row, Tree &col) {
	int ans = INF;
	int n = row.n;
	int m = col.n;

	if (min(row.get_ma(r1, r1 + 1), col.get_ma(c1, c1 + 1)) > k)
		return INF;
	if (min(row.get_ma(r2, r2 + 1), col.get_ma(c2, c2 + 1)) > k)
		return INF;

	if (r1 > r2) {
		swap(r1, r2);
		swap(c1, c2);
	}
	auto p1 = row.find_range(r1, k);
	auto p2 = row.find_range(r2, k);
	if (col.get_ma(c1, c1 + 1) <= k)
		p1 = Seg(0, n);
	if (col.get_ma(c2, c2 + 1) <= k)
		p2 =Seg(0, n);
	
	Seg p = p1.isect(p2);
	if (p.r <= p.l)
		return INF;

	if (col.get_ma(min(c1, c2) + 1, max(c1, c2)) <= k)
		return abs(r1 - r2) + abs(c1 - c2);

	Seg ra1 = p.isect(Seg(r1, r2 + 1));

	if (ra1.l < ra1.r && row.get_mi(ra1.l, ra1.r) <= k)
		return abs(r1 - r2) + abs(c1 - c2);
	
	int ind1 = row.find_first_less(r1, k), ind2 = row.find_first_gr(r2, k);

	if (p1.in(ind1) && p2.in(ind1))
		ans = min(ans, abs(ind1 - r1) + abs(ind1 - r2) + abs(c1 - c2));
	if (p1.in(ind2) && p2.in(ind2))
		ans = min(ans, abs(ind2 - r1) + abs(ind2 - r2) + abs(c1 - c2));

	return ans;
}

int main() {
#ifdef LOCAL
	freopen("input.txt", "r", stdin);
	freopen("output.txt", "w", stdout);
	auto ti = clock();
#endif
	ios::sync_with_stdio(0);
	cin.tie(0);
	cout.tie(0);

	int n, m, q;
	cin >> n >> m >> q;

	Tree row1(n), col1(m), row2(m), col2(n);

	while (q--) {
		row1.d++;
		col1.d++;
		row2.d++;
		col2.d++;

		/*for (int i = 0; i < n; i++) {
			for (int j = 0; j < m; j++)
				cout << min(row1.val[i], col1.val[j]) + row1.d << ' ';
			cout << endl;
		}
		cout << endl;
		for (int i = 0; i < m; i++) {
			for (int j = 0; j < n; j++)
				cout << min(row2.val[i], col2.val[j]) + row1.d << ' ';
			cout << endl;
		}
		cout << "-------------------\n";*/

		int t;
		cin >> t;
		if (t == 1) {
			int r;
			cin >> r;
			r--;
			row1.mod(r);
			col2.mod(r);
		}
		else if (t == 2) {
			int c;
			cin >> c;
			c--;
			col1.mod(c);
			row2.mod(c);
		}
		else {
			int r1, c1, r2, c2, k;
			cin >> r1 >> c1 >> r2 >> c2 >> k;

			r1--, r2--, c1--, c2--;

			int a = get_ver(r1, c1, r2, c2, k, row1, col1);
			int b = get_ver(c1, n - 1 - r1, c2, n - r2 - 1, k, row2, col2);
			
			int c = min(a, b);

			if (c == INF)
				cout << -1 << '\n';
			else
				cout << c << "\n";
		}
		/*row.print(0, 0, n);
		cout << endl;
		col.print(0, 0, m);
		cout << endl << endl;*/
	}

#ifdef LOCAL
	cout << "\nExecuted in "
		<< fixed << setprecision(5) << (ld)(clock() - ti) / (ld)CLOCKS_PER_SEC << " seconds" << endl;
#endif
	return 0;
}