#include <iostream>
#include <iomanip>
#include <vector>
#include <string>
#include <queue>
#include <set>
#include <map>
#include <stack>
#include <bitset>
#include <algorithm>
#include <cmath>
#include <ctime>
#include <cstring>
#include <chrono>
#include <random>

using namespace std;

#define int long long
#define sz(a) (int)(a.size())
#define pb emplace_back

void fast_io() {
	ios_base::sync_with_stdio(false);
	cin.tie(0);
	cout.tie(0);
}

struct Node {
	int l, r, mn = 0, mx = 0, delta = 0;
	Node *left = nullptr, *right = nullptr;
	Node(int l_, int r_) : l(l_), r(r_) {};
};

const int INF = 1e9 + 228;

void push(Node *node) {
	if (node->left) {
		node->left->delta += node->delta;
		node->left->mn += node->delta;
		node->right->delta += node->delta;
		node->right->mn += node->delta;
		node->left->mx += node->delta;
		node->right->mx += node->delta;
	}
	node->delta = 0;
	return;
}

Node *build_tree(int l, int r) {
	Node *node = new Node(l, r);
	if (l == r) {
		return node;
	}
	int m = (l + r) / 2;
	node->left = build_tree(l, m);
	node->right = build_tree(m + 1, r);
	return node;
}

void add_all(Node *node) {
	push(node);
	node->mn++;
	node->mx++;
	node->delta++;
}

void get_null(Node *node, int i) {
	push(node);
	if (node->l == node->r) {
		node->mn = node->mx = 0;
		return;
	}
	if (node->left->r >= i) {
		get_null(node->left, i);
	}
	else {
		get_null(node->right, i);
	}
	node->mn = min(node->left->mn, node->right->mn);
	node->mx = max(node->left->mx, node->right->mx);
	return;
}

int get_i(Node *node, int i) {
	while (node->l != node->r) {
		push(node);
		if (node->left->r >= i) {
			node = node->left;
		}
		else {
			node = node->right;
		}
	}
	return node->mn;
}

int get_suf_less(Node *node, int l, int k) {
	push(node);
	if (node->l == node->r) {
		return l;
	}
	int ans = INF;
	if (node->left->r >= l && node->left->mn <= k) {
		ans = get_suf_less(node->left, l, k);
	}
	if (ans == INF && node->right->mn <= k) {
		ans = get_suf_less(node->right, node->right->l, k);
	}
	return ans;
}

int get_pref_less(Node *node, int r, int k) {
	push(node);
	if (node->l == node->r) {
		return node->l;
	}
	int ans = -1;
	if (node->right->l <= r && node->right->mn <= k) {
		ans = get_pref_less(node->right, r, k);
	}
	if (ans == -1 && node->left->mn <= k) {
		ans = get_pref_less(node->left, node->left->r, k);
	}
	return ans;
}

int get_mx(Node *node, int l, int r) {
	push(node);
	if (node->l == l && node->r == r) {
		return node->mx;
	}
	int ans = -1;
	if (node->left->r >= l) {
		ans = max(ans, get_mx(node->left, l, min(node->left->r, r)));
	}
	if (node->right->l <= r) {
		ans = max(ans, get_mx(node->right, max(l, node->right->l), r));
	}
	return ans;
}

signed main() {
	fast_io();
	int n, m, q;
	cin >> n >> m >> q;
	Node *rows = build_tree(0, n - 1);
	Node *cols = build_tree(0, m - 1);
	for (int i = 0; i < q; i++) {
		add_all(rows);
		add_all(cols);
		int q;
		cin >> q;
		if (q == 1) {
			int r;
			cin >> r;
			r--;
			get_null(rows, r);
		}
		if (q == 2) {
			int c;
			cin >> c;
			c--;
			get_null(cols, c);
		}
		if (q == 3) {
			int r_a, c_a, r_b, c_b, k;
			cin >> r_a >> c_a >> r_b >> c_b >> k;
			r_a--;
			r_b--;
			c_a--;
			c_b--;
			bool row_a = get_i(rows, r_a) <= k, col_a = get_i(cols, c_a) <= k, 
				row_b = get_i(rows, r_b) <= k, col_b = get_i(cols, c_b) <= k;
			if ((row_a && col_b) || (row_b && col_a)) {
				cout << abs(r_b - r_a) + abs(c_b - c_a) << '\n';
			}
			else if ((col_a && col_b) || (row_a && row_b)) {
				if (col_a && col_b) {
					if (get_mx(cols, min(c_a, c_b), max(c_a, c_b)) <= k) {
						cout << abs(r_b - r_a) + abs(c_b - c_a) << '\n';
					}
					else {
						int pref_less = get_pref_less(rows, max(r_a, r_b), k);
						int suf_less = get_suf_less(rows, min(r_a, r_b), k);
						if (pref_less >= min(r_a, r_b) || suf_less <= max(r_a, r_b)) {
							cout << abs(r_b - r_a) + abs(c_b - c_a) << '\n';
						}
						else {
							int v1 = INF, v2 = INF;
							if (pref_less != -1) {
								v1 = abs(r_a - pref_less) + abs(r_b - pref_less) + abs(c_b - c_a);
							}
							if (suf_less != INF) {
								v2 = abs(r_a - suf_less) + abs(r_b - suf_less) + abs(c_b - c_a);
							}
							if (v1 != INF || v2 != INF) {
								cout << min(v1, v2) << '\n';
							}
							else {
								cout << -1 << '\n';
							}
						}
					}
				}
				else {
					if (get_mx(cols, min(r_a, r_b), max(r_a, r_b)) <= k) {
						cout << abs(r_b - r_a) + abs(c_b - c_a) << '\n';
					}
					else {
						int pref_less = get_pref_less(cols, max(c_a, c_b), k);
						int suf_less = get_suf_less(cols, min(c_a, c_b), k);
						if (pref_less >= min(c_a, c_b) || suf_less <= max(c_a, c_b)) {
							cout << abs(r_b - r_a) + abs(c_b - c_a) << '\n';
						}
						else {
							int v1 = INF, v2 = INF;
							if (pref_less != -1) {
								v1 = abs(c_a - pref_less) + abs(c_b - pref_less) + abs(r_b - r_a);
							}
							if (suf_less != INF) {
								v2 = abs(c_a - suf_less) + abs(c_b - suf_less) + abs(r_b - r_a);
							}
							if (v1 != INF || v2 != INF) {
								cout << min(v1, v2) << '\n';
							}
							else {
								cout << -1 << '\n';
							}
						}
					}
				}
			}
			else {
				cout << -1 << '\n';
			}

		}
	}
	return 0;
}