#include <iostream>
#include <iomanip>
#include <vector>
#include <string>
#include <queue>
#include <set>
#include <map>
#include <stack>
#include <bitset>
#include <algorithm>
#include <cmath>
#include <ctime>
#include <cstring>
#include <chrono>
#include <random>

using namespace std;

#define int long long
#define sz(a) (int)(a.size())
#define pb emplace_back

void fast_io() {
	ios_base::sync_with_stdio(false);
	cin.tie(0);
	cout.tie(0);
}

const int MAXN = 50;
int field[MAXN][MAXN];
int dist[MAXN][MAXN];

struct Node {
	int l, r, mn = 0, mx = 0, delta = 0;
	Node *left = nullptr, *right = nullptr;
	Node(int l_, int r_) : l(l_), r(r_) {};
};

const int INF = 1e18 + 228;
int n, m, q;
vector<vector<int>> queries;

void push(Node *node) {
	if (node->left) {
		node->left->delta += node->delta;
		node->left->mn += node->delta;
		node->right->delta += node->delta;
		node->right->mn += node->delta;
		node->left->mx += node->delta;
		node->right->mx += node->delta;
	}
	node->delta = 0;
	return;
}

Node *build_tree(int l, int r) {
	Node *node = new Node(l, r);
	if (l == r) {
		return node;
	}
	int m = (l + r) / 2;
	node->left = build_tree(l, m);
	node->right = build_tree(m + 1, r);
	return node;
}

void add_all(Node *node) {
	node->mn++;
	node->mx++;
	node->delta++;
	push(node);
}

void get_null(Node *node, int i) {
	push(node);
	if (node->l == node->r) {
		node->mn = node->mx = 0;
		return;
	}
	if (node->left->r >= i) {
		get_null(node->left, i);
	}
	else {
		get_null(node->right, i);
	}
	node->mn = min(node->left->mn, node->right->mn);
	node->mx = max(node->left->mx, node->right->mx);
	return;
}

int get_i(Node *node, int i) {
	while (node->l != node->r) {
		push(node);
		if (node->left->r >= i) {
			node = node->left;
		}
		else {
			node = node->right;
		}
	}
	push(node);
	return node->mn;
}

int get_suf_less(Node *node, int l, int k) {
	push(node);
	if (node->l == node->r) {
		if (node->mn <= k) {
			return node->l;
		}
		else {
			return INF;
		}
	}
	int ans = INF;
	if (node->left->r >= l && node->left->mn <= k) {
		ans = get_suf_less(node->left, l, k);
	}
	if (ans == INF && node->right->mn <= k) {
		ans = get_suf_less(node->right, max(l, node->right->l), k);
	}
	return ans;
}

int get_pref_less(Node *node, int r, int k) {
	push(node);
	if (node->l == node->r) {
		if (node->mn <= k) {
			return node->l;
		}
		else {
			return -1;
		}
	}
	int ans = -1;
	if (node->right->l <= r && node->right->mn <= k) {
		ans = get_pref_less(node->right, r, k);
	}
	if (ans == -1 && node->left->mn <= k) {
		ans = get_pref_less(node->left, min(r, node->left->r), k);
	}
	return ans;
}

int get_mx(Node *node, int l, int r) {
	push(node);
	if (node->l == l && node->r == r) {
		return node->mx;
	}
	int ans = -1;
	if (node->left->r >= l) {
		ans = max(ans, get_mx(node->left, l, min(node->left->r, r)));
	}
	if (node->right->l <= r) {
		ans = max(ans, get_mx(node->right, max(l, node->right->l), r));
	}
	return ans;
}

vector<int> clever() {
	vector<int> out;
	Node *rows = build_tree(0, n - 1);
	Node *cols = build_tree(0, m - 1);
	for (int i = 0; i < q; i++) {
		add_all(rows);
		add_all(cols);
		int cur = queries[i][0];
		if (cur == 1) {
			int r = queries[i][1];
			r--;
			get_null(rows, r);
		}
		if (cur == 2) {
			int c = queries[i][1];
			c--;
			get_null(cols, c);
		}
		if (cur == 3) {
			int r_a = queries[i][1], c_a = queries[i][2], r_b = queries[i][3], c_b = queries[i][4], k = queries[i][5];
			r_a--;
			r_b--;
			c_a--;
			c_b--;
			bool row_a = (get_i(rows, r_a) <= k), col_a = (get_i(cols, c_a) <= k),
				row_b = (get_i(rows, r_b) <= k), col_b = (get_i(cols, c_b) <= k);
			if ((row_a && col_b) || (row_b && col_a)) {
				out.pb(abs(r_b - r_a) + abs(c_b - c_a));
			}
			else if ((col_a && col_b) || (row_a && row_b)) {
				if (col_a && col_b) {
					if (get_mx(cols, min(c_a, c_b), max(c_a, c_b)) <= k) {
						out.pb(abs(r_b - r_a) + abs(c_b - c_a));
					}
					else {
						int pref_less = get_pref_less(rows, max(r_a, r_b), k);
						int suf_less = get_suf_less(rows, min(r_a, r_b), k);
						if (pref_less >= min(r_a, r_b) || suf_less <= max(r_a, r_b)) {
							out.pb(abs(r_b - r_a) + abs(c_b - c_a));
						}
						else {
							int v1 = INF, v2 = INF;
							if (pref_less != -1) {
								v1 = abs(r_a - pref_less) + abs(r_b - pref_less) + abs(c_b - c_a);
							}
							if (suf_less != INF) {
								v2 = abs(r_a - suf_less) + abs(r_b - suf_less) + abs(c_b - c_a);
							}
							if (v1 != INF || v2 != INF) {
								out.pb(min(v1, v2));
							}
							else {
								out.pb(-1);
							}
						}
					}
				}
				else {
					if (get_mx(rows, min(r_a, r_b), max(r_a, r_b)) <= k) {
						out.pb(abs(r_b - r_a) + abs(c_b - c_a));
					}
					else {
						int pref_less = get_pref_less(cols, max(c_a, c_b), k);
						int suf_less = get_suf_less(cols, min(c_a, c_b), k);
						if (pref_less >= min(c_a, c_b) || suf_less <= max(c_a, c_b)) {
							out.pb(abs(r_b - r_a) + abs(c_b - c_a));
						}
						else {
							int v1 = INF, v2 = INF;
							if (pref_less != -1) {
								v1 = abs(c_a - pref_less) + abs(c_b - pref_less) + abs(r_b - r_a);
							}
							if (suf_less != INF) {
								v2 = abs(c_a - suf_less) + abs(c_b - suf_less) + abs(r_b - r_a);
							}
							if (v1 != INF || v2 != INF) {
								out.pb(min(v1, v2));
							}
							else {
								out.pb(-1);
							}
						}
					}
				}
			}
			else {
				out.pb(-1);
			}
		}
	}
	return out;
}

vector<pair<int, int>> mask = { {-1, 0}, {1, 0}, {0, 1}, {0, -1} };


bool correct(int x, int y) {
	return x >= 0 && x < n && y >= 0 && y < m;
}

void bfs(int x, int y, int k) {
	dist[x][y] = 0;
	if (field[x][y] <= k) {
		queue<pair<int, int>> q;
		q.push(make_pair(x, y));
		while (sz(q)) {
			pair<int, int> v = q.front();
			q.pop();
			for (auto d : mask) {
				if (dist[v.first + d.first][v.second + d.second] == INF &&
					correct(v.first + d.first, v.second + d.second) &&
					field[v.first + d.first][v.second + d.second] <= k) {
					dist[v.first + d.first][v.second + d.second] = dist[v.first][v.second] + 1;
					q.push(make_pair(v.first + d.first, v.second + d.second));
				}
			}
		}
	}
}

vector<int> dumb() {
	vector<int> out;
	for (int i = 0; i < q; i++) {
		int cur = queries[i][0];
		for (int j = 0; j < n; j++) {
			fill(dist[j], dist[j] + 50, INF);
			for (int k = 0; k < m; k++) {
				field[j][k]++;
			}
		}
		if (cur == 1) {
			int r = queries[i][1];
			r--;
			for (int j = 0; j < m; j++) {
				field[r][j] = 0;
			}
		}
		if (cur == 2) {
			int c = queries[i][1];
			c--;
			for (int j = 0; j < n; j++) {
				field[j][c] = 0;
			}
		}
		if (cur == 3) {
			int r_a = queries[i][1], c_a = queries[i][2], r_b = queries[i][3], c_b = queries[i][4], k = queries[i][5];
			r_a--;
			c_a--;
			r_b--;
			c_b--;
			bfs(r_a, c_a, k);
			if (dist[r_b][c_b] == INF) {
				out.pb(-1);
			}
			else {
				out.pb(dist[r_b][c_b]);
			}
		}
	}
	return out;
}

signed main() {
	fast_io();
	int tests = 1;
//	cin >> tests;
	for (int t = 0; t < tests; t++) {
		cin >> n >> m >> q;
		queries.resize(q);
		for (int i = 0; i < q; i++) {
			int cur;
			cin >> cur;
			queries[i].pb(cur);
			if (cur == 1 || cur == 2) {
				int x;
				cin >> x;
				queries[i].pb(x);
			}
			else {
				for (int j = 0; j < 5; j++) {
					int x;
					cin >> x;
					queries[i].pb(x);
				}
			}
		}
		vector<int> v1 = clever();// , v2 = dumb();
		for (auto el : v1) {
			cout << el << '\n';
		}
		//if (v1 != v2) {
		//	int kek = 228;
		//}
		//queries.clear();
		//memset(dist, 0, sizeof dist);
		//memset(field, 0, sizeof field);
	}
	return 0;
}