import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.util.*;

public class A {

    public static class Segment {
        public int l, r, color;

        public Segment(int l, int r, int color) {
            this.l = l;
            this.r = r;
            this.color = color;
        }
    }

    public static class Event implements Comparable<Event> {
        public int x;
        public int type;
        public int id;

        public static final int CLOSE = 0;
        public static final int OPEN = 1;


        public Event(int x, int type, int id) {
            this.x = x;
            this.type = type;
            this.id = id;
        }

        @Override
        public int compareTo(Event e) {
            if (e.x != x) {
                return Integer.compare(x, e.x);
            } else {
                return Integer.compare(type, e.type);
            }
        }
    }

    public static class Pair implements Comparable<Pair> {
        public int first, second;

        @Override
        public int compareTo(Pair p) {
            if (p.first != first) {
                return Integer.compare(first, p.first);
            } else {
                return Integer.compare(second, p.second);
            }
        }

        public Pair(int first, int ind) {
            this.first = first;
            this.second = ind;
        }
    }

    public void solve() throws Exception {
        int cnt = nextInt();
        int colors = nextInt();
        int[] left = new int[colors + 1];
        int[] right = new int[colors + 1];
        Arrays.fill(left, -1);
        Arrays.fill(right, -1);
        int[] a = new int[cnt];
        for (int i = 0; i < cnt; i++) {
            a[i] = nextInt();
        }
        for (int i = 0; i < cnt; i++) {
            if (left[a[i]] == -1) {
                left[a[i]] = i;
                right[a[i]] = i;
            } else {
                right[a[i]] = i;
            }
        }


        Segment[] segments = new Segment[colors + 1];

        for (int i = 1; i <= colors; i++) {
            if (left[i] != -1) {
                segments[i] = new Segment(left[i], right[i], i);
            }
        }

        List<Event> events = new ArrayList<>(segments.length * 2);
        for (int i = 1; i <= colors; i++) {
            if (segments[i] != null) {
                events.add(new Event(segments[i].l, Event.OPEN, i));
                events.add(new Event(segments[i].r, Event.CLOSE, i));
            }
        }
        Collections.sort(events);
        List<Segment>[] segByBal = new List[segments.length + 1];
        for (int i = 0; i < segments.length + 1; i++) {
            segByBal[i] = new ArrayList<>();
        }
        int bal = 0;
        for (int i = 0; i < events.size(); i++) {
            Event e = events.get(i);
            Segment seg = segments[e.id];
            if (e.type == Event.OPEN) {
                bal++;
                segByBal[bal].add(seg);
            } else {
                bal--;
            }
        }
        List<Segment> ansL = new ArrayList<>((int) 3e5);
        int[] timeByC = new int[colors + 1];
        for (int i = 0; i < segByBal.length; i++) {
            for (int j = 0; j < segByBal[i].size(); j++) {
                Segment s = segByBal[i].get(j);
                timeByC[s.color] = ansL.size();
                ansL.add(s);
            }
        }

        List<Integer>[] st = new List[cnt];
        for (int i = 0; i < cnt; i++) {
            st[i] = new ArrayList<>();
        }

        for (int i = 1; i <= colors; i++) {
            if (left[i] != -1) {
                st[left[i]].add(i);
                st[right[i]].add(-i);
            }
        }

        Stack<Pair> stack = new Stack<>();
        stack.add(new Pair(-1, -1));
        for (int i = 0; i < cnt; i++) {
            boolean pop = false;
            for (int j = 0; j < st[i].size(); j++) {
                int cur = st[i].get(j);
                int curTime = timeByC[Math.abs(cur)];
                if (cur > 0 && curTime > stack.peek().second) {
                    stack.push(new Pair(cur, curTime));
                } else if (cur < 0 && Math.abs(cur) == stack.peek().first) {
                    stack.pop();
                    pop = true;
                }
            }
            if (pop) {
                continue;
            }
            if (a[i] != stack.peek().first && stack.peek().first != -1) {
                out.println(-1);
                return;
            }
        }
        out.println(ansL.size());
        for (int i = 0; i < ansL.size(); i++) {
            Segment s = ansL.get(i);
            out.print(s.color);
            out.print(' ');
            out.print(s.l + 1);
            out.print(' ');
            out.println(s.r + 1);
        }
    }

    public void run() throws Exception {
        br = new BufferedReader(new InputStreamReader(System.in));
        out = new PrintWriter(System.out);
        solve();
        br.close();
        out.close();
    }

    BufferedReader br;
    StringTokenizer st;
    PrintWriter out;

    public String nextToken() throws Exception {
        while (st == null || !st.hasMoreTokens()) {
            st = new StringTokenizer(br.readLine());
        }
        return st.nextToken();
    }

    public int nextInt() throws Exception {
        return Integer.parseInt(nextToken());
    }

    public static void main(String[] args) throws Exception {
        new A().run();
    }
}
