#include <iostream>
#include <map>
#include <set>
#include <vector>
#include <cmath>
#include <iomanip>
#include <algorithm>
#include <new>

using namespace std;

typedef long long ll;

const int SZ = 1e6 + 100;
const int INF = 1e8;

struct der
{
    der *l, *r;
    int mn, mx;
    der()
    {
        l = NULL;
        r = NULL;
        mx = 0;
        mn = 0;
    }
};

typedef der* pder;

int stolb[SZ] = {};
int strok[SZ] = {};
int n, m, q;
int k;

pder d[2];

int val;

void calc(pder ver)
{
    if (ver->l && ver->r)
    {
        ver->mx = -INF;
        ver->mn = INF;
    }
    if (ver->l)
    {
        ver->mx = max(ver->mx, ver->l->mx);
        ver->mn = min(ver->mn, ver->l->mn);
    }
    if (ver->r)
    {
        ver->mx = max(ver->mx, ver->r->mx);
        ver->mn = min(ver->mn, ver->r->mn);
    }
}

int get_max(pder ver, int tl, int tr, int l, int r)
{
    if (ver == NULL) return 0;
    if (tl == l && tr == r)
    {
        return ver->mx;
    }
    int sr = (tl + tr) / 2;
    if (r <= sr)
    {
        return get_max(ver->l, tl, sr, l, r);
    }
    if (l > sr)
    {
        return get_max(ver->r, sr + 1, tr, l, r);
    }
    return max(get_max(ver->l, tl, sr, l, sr),
               get_max(ver->r, sr + 1, tr, sr + 1, r));
}

int get_min(pder ver, int tl, int tr, int l, int r)
{
    if (ver == NULL) return 0;
    if (tl == l && tr == r)
    {
        return ver->mn;
    }
    int sr = (tl + tr) / 2;
    if (r <= sr)
    {
        return get_min(ver->l, tl, sr, l, r);
    }
    if (l > sr)
    {
        return get_min(ver->r, sr + 1, tr, l, r);
    }
    return min(get_min(ver->l, tl, sr, l, sr),
               get_min(ver->r, sr + 1, tr, sr + 1, r));
}

void d_add(pder ver, int tl, int tr, int ind)
{
    if (tl == tr)
    {
        ver->mn = val;
        ver->mx = val;
        return;
    }
    int sr = (tl + tr) / 2;
    if (ind <= sr)
    {
        if (!ver->l)
        {
            ver->l = new(der);
        }
        d_add(ver->l, tl, sr, ind);
    }
    else
    {
        if (!ver->r)
        {
            ver->r = new(der);
        }
        d_add(ver->r, sr + 1, tr, ind);
    }
    calc(ver);
}


int dist(int x1, int y1, int x2, int y2)
{
    return abs(x1 - x2) + abs(y1 - y2);
}

int peres(int x1, int y1, int x2, int y2)
{
    if (strok[y1] > k || stolb[x2] > k) return INF;
    return dist(x1, y1, x2, y2);
}

int get_strok(int x1, int y1, int x2, int y2)
{
    int can;
    if (strok[y1] > k || strok[y2] > k) return INF;
    can = get_max(d[0], 0, n - 1, min(y1, y2), max(y1, y2));
    if (can <= k) return dist(x1, y1, x2, y2);

    int val;
    val = INF;
    int from, to;
    from = min(x1, x2);
    to = max(x1, x2);

    can = get_min(d[1], 0, m - 1, from, to);
    if (can <= k) return dist(x1, y1, x2, y2);

    int gl, gr, mid;

    gl = 0;
    gr = from - 1;
    while (gl < gr)
    {
        mid = (gl + gr + 1) / 2;
        can = get_min(d[1], 0, m - 1, mid, from - 1);
        if (can <= k) gl = mid;
        else gr = mid - 1;
    }
    can = get_min(d[1], 0, m - 1, gl, from - 1);
    if (can <= k) val = min(val, abs(x1 - gl) + abs(y2 - y1) + abs(x2 - gl));

    gl = to + 1;
    gr = n - 1;
    while (gl < gr)
    {
        mid = (gl + gr) / 2;
        can = get_min(d[1], 0, m - 1, to + 1, mid);
        if (can <= k) gr = mid;
        else gl = mid + 1;
    }
    can = get_min(d[1], 0, m - 1, to + 1, gl);
    if (can <= k) val = min(val, abs(x1 - gl) + abs(y2 - y1) + abs(x2 - gl));

    return val;
}

int get_stolb(int x1, int y1, int x2, int y2)
{
    if (stolb[x1] > k || stolb[x2] > k) return INF;
    int can = get_max(d[1], 0, m - 1, min(x1, x2), max(x1, x2));
    if (can <= k) return dist(x1, y1, x2, y2);

    int val;
    val = INF;
    for (int i = 0; i < n; i++)
    {
        if (strok[i] <= k)
        {
            val = min(val, abs(y1 - i) + abs(x2 - x1) + abs(y2 - i));
        }
    }
    return val;
}


int main()
{
    ios::sync_with_stdio(false);

    d[0] = new(der);
    d[1] = new(der);

    cin >> n >> m >> q;
    int sim;
    int x1, x2, y1, y2;
    for (int i = 1; i <= q; i++)
    {
        cin >> sim;
        if (sim == 1)
        {
            cin >> y1;
            y1--;
            strok[y1] = -i;
            val = -i;
            d_add(d[0], 0, n - 1, y1);
            continue;
        }
        if (sim == 2)
        {
            cin >> x1;
            x1--;
            stolb[x1] = -i;
            val = -i;
            d_add(d[1], 0, m - 1, x1);
            continue;
        }
        if (sim == 3)
        {
            cin >> y1 >> x1 >> y2 >> x2 >> k;
            k -= i;
            x1--;
            y1--;
            x2--;
            y2--;
            bool fl1, fl2;
            fl1 = false;
            fl2 = false;
            if (strok[y1] == 0 && stolb[x1] == 0) fl1 = true;
            if (strok[y2] == 0 && stolb[x2] == 0) fl2 = true;
            if (fl1 || fl2)
            {
                if (k >= 0) cout << dist(x1, y1, x2, y2) << "\n";
                else cout << -1 << "\n";
                continue;
            }

            int ans = INF;
            int zn = max(min(strok[y1], stolb[x1]),
                         min(strok[y2], stolb[x2]));
            if (zn > k)
            {
                cout << -1 << "\n";
                continue;
            }

            if (strok[y1] < 0 && stolb[x2] < 0)
            {
                ans = min(ans, peres(x1, y1, x2, y2));
            }
            if (strok[y2] < 0 && stolb[x1] < 0)
            {
                ans = min(ans, peres(x2, y2, x1, y1));
            }
            if (strok[y1] < 0 && strok[y2] < 0)
            {
                ans = min(ans, get_strok(x1, y1, x2, y2));
            }
            if (stolb[x1] < 0 && stolb[x2] < 0)
            {
                ans = min(ans, get_stolb(x1, y1, x2, y2));
            }

            if (ans < INF) cout << ans << "\n";
            else cout << -1 << "\n";

            continue;
        }
    }

    return 0;
}
