#include <iostream>
#include <ctime>
#include <vector>
#include <iomanip>
#include <cstring>
#include <climits>
#include <cassert>

using namespace std;
struct Node {
    int min = INT_MAX, max = INT_MIN;
};
int n, m, q;
const int MAXN = (int) (1e6 + 7);
int rows[MAXN];
int columns[MAXN];
Node st_col[4 * MAXN];
Node st_row[4 * MAXN];

void set(Node *a, int x, int l, int r, int pos, int many) {
    if (pos < l || r <= pos)
        return;
    if (r - l == 1) {
        a[x].min = a[x].max = many;
        return;
    }
    int m = (l + r) / 2;
    set(a, x * 2, l, m, pos, many);
    set(a, x * 2 + 1, m, r, pos, many);
    a[x].min = min(a[x * 2].min, a[x * 2 + 1].min);
    a[x].max = max(a[x * 2].max, a[x * 2 + 1].max);
}

int get(Node *a, int x, int l, int r, int L, int R) {
    if (L >= r || l >= R)
        return INT_MAX;
    if (L <= l && r <= R)
        return a[x].min;
    int m = (l + r) / 2;
    return min(get(a, x * 2, l, m, L, R), get(a, x * 2 + 1, m, r, L, R));
}

int get_left(Node *a, int x, int l, int r, int L, int R, int many) {
    if (L >= r || l >= R || a[x].max < many)
        return INT_MAX;
    if (r - l == 1)
        return l;
    int m = (l + r) / 2;
    int ret = get_left(a, x * 2, l, m, L, R, many);
    if (ret != INT_MAX)
        return ret;
    return get_left(a, x * 2 + 1, m, r, L, R, many);
}

int get_right(Node *a, int x, int l, int r, int L, int R, int many) {
    if (L >= r || l >= R || a[x].max < many)
        return -1;
    if (r - l == 1)
        return l;
    int m = (l + r) / 2;
    int ret = get_right(a, x * 2 + 1, m, r, L, R, many);
    if (ret != -1)
        return ret;
    return get_right(a, x * 2, l, m, L, R, many);
}

inline bool good_row(int r, int t, int k) {
    return t - rows[r] <= k;
}

inline bool good_col(int c, int t, int k) {
    return t - columns[c] <= k;
}

inline void solve() {
    memset(rows, -1, sizeof(int) * MAXN);
    memset(columns, -1, sizeof(int) * MAXN);
    int N = 1, M = 1;
    while (N < n) N *= 2;
    while (M < m) M *= 2;
//    assert(n == m);
    for (int i = 0; i < n; i++)
        set(st_row, 1, 0, N, i, -1);
    for (int i = 0; i < m; i++)
        set(st_col, 1, 0, M, i, -1);
    for (int i = 0; i < q; i++) {
        int type;
        cin >> type;
        if (type == 1) {
            int r;
            cin >> r;
            rows[r - 1] = i;
            set(st_row, 1, 0, N, r - 1, i);
        }
        if (type == 2) {
            int c;
            cin >> c;
            columns[c - 1] = i;
            set(st_col, 1, 0, M, c - 1, i);
        }
        if (type == 3) {
            int x1, y1, x2, y2, k;
            cin >> x1 >> y1 >> x2 >> y2 >> k;
            x1--, y1--, x2--, y2--;
            int ans = INT_MAX;
            if (get(st_row, 1, 0, N, min(x1, x2), max(x1, x2) + 1) >= i - k ||
                get(st_col, 1, 0, M, min(y1, y2), max(y2, y1) + 1) >= i - k ||
                (good_row(x1, i, k) && good_col(y2, i, k)) || (good_row(x2, i, k) && good_col(y1, i, k))) {
                cout << abs(x1 - x2) + abs(y1 - y2) << '\n';
                continue;
            }

            if (good_row(x1, i, k) && good_row(x2, i, k)) {
                int ret = get_left(st_col, 1, 0, M, min(y1, y2), m, i - k);
                if (ret != INT_MAX)
                    ans = min(ans, abs(x1 - x2) + abs(y1 - ret) + abs(y2 - ret));
                ret = get_right(st_col, 1, 0, M, 0, min(y1, y2) + 1, i - k);
                if (ret != -1)
                    ans = min(ans, abs(x1 - x2) + abs(y1 - ret) + abs(y2 - ret));
            }
            if (good_col(y1, i, k) && good_col(y2, i, k)) {
                int ret = get_left(st_row, 1, 0, N, min(x1, x2), n, i - k);
                if (ret != INT_MAX)
                    ans = min(ans, abs(y1 - y2) + abs(x1 - ret) + abs(x2 - ret));
                ret = get_right(st_row, 1, 0, N, 0, min(x1, x2) + 1, i - k);
                if (ret != -1)
                    ans = min(ans, abs(y1 - y2) + abs(x1 - ret) + abs(x2 - ret));
            }
            if (ans == INT_MAX)
                cout << "-1\n";
            else
                cout << ans << '\n';
        }
    }
}

int main() {
#ifdef LOCAL
    freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);
#else
    ios::sync_with_stdio(0);
    cin.tie(0), cout.tie(0);
#endif

    while (cin >> n >> m >> q) {
#ifdef LOCAL
        clock_t st = clock();
        solve();
        cerr << setprecision(4) << fixed << 1. * (clock() - st) / CLOCKS_PER_SEC << '\n';
#else
        solve();
#endif
    }
}