#include <algorithm>
#include <iostream>
#include <fstream>
#include <vector>
#include <string>

using namespace std;

const int BLOCK_SIZE = 1024;

struct Block {
	vector<int> a;
	int mx, mn;

	Block() {
		a.resize(BLOCK_SIZE);
		mx = mn = 0;
	}

	void update() {
		mx = a[0];
		mn = a[0];
		for (auto i : a) {
			if (i > mx) {
				mx = i;
			}
			if (i < mn) {
				mn = i;
			}
		}
	}
};

int get(vector<Block>& a, int i) {
	return a[i / BLOCK_SIZE].a[i % BLOCK_SIZE];
}

int count(vector<Block>& a, int left, int right, int k) {
	int result = 0;
	for (; left <= right && (left % BLOCK_SIZE); ++left) {
		if (get(a, left) <= k) {
			++result;
		}
	}
	for (; right >= left && (right % BLOCK_SIZE); --right) {
		if (get(a, right) <= k) {
			++result;
		}
	}
	if (right >= left) {
		if (get(a, right) <= k) {
			++result;
		}
	}
	for (int i = left / BLOCK_SIZE; i < right / BLOCK_SIZE; ++i) {
		if (a[i].mx <= k) {
			result += BLOCK_SIZE;
		}
		else {
			if (a[i].mn <= k) {
				result += 1;
			}
		}
	}
	return result;
}

int main() {
#ifdef _DEBUG
	ifstream cin("input.txt");
	ofstream cout("output.txt");
#else
	ios_base::sync_with_stdio(false);
	cout.tie(nullptr);
	cin.tie(nullptr);
#endif

	int n, m, q;
	cin >> n >> m >> q;
	vector<Block> x((n - 1) / BLOCK_SIZE + 1);
	vector<Block> y((m - 1) / BLOCK_SIZE + 1);
	for (int time = 1; time <= q; ++time) {
		int type;
		cin >> type;
		if (type == 1) {
			int xx;
			cin >> xx;
			--xx;
			x[xx / BLOCK_SIZE].a[xx % BLOCK_SIZE] = -time;
			x[xx / BLOCK_SIZE].update();
		}
		if (type == 2) {
			int yy;
			cin >> yy;
			--yy;
			y[yy / BLOCK_SIZE].a[yy % BLOCK_SIZE] = -time;
			y[yy / BLOCK_SIZE].update();
		}
		if (type == 3) {
			int x1, x2, y1, y2, k;
			cin >> x1 >> y1 >> x2 >> y2 >> k;
			--x1;
			--x2;
			--y1;
			--y2;
			k = time - k;
			if (get(x, x1) + k > 0 && get(y, y1) + k > 0) {
				cout << -1 << "\n";
				continue;
			}
			if (get(x, x2) + k > 0 && get(y, y2) + k > 0) {
				cout << -1 << "\n";
				continue;
			}
			if (get(x, x1) + k <= 0 && get(y, y2) + k <= 0) {
				cout << (abs(x1 - x2) + abs(y1 - y2)) << "\n";
				continue;
			}
			if (get(x, x2) + k <= 0 && get(y, y1) + k <= 0) {
				cout << (abs(x1 - x2) + abs(y1 - y2)) << "\n";
				continue;
			}
			if (get(x, x1) + k <= 0 && get(x, x2) + k <= 0) {
				if (count(x, min(x1, x2), max(x1, x2), -k) == abs(x1 - x2) + 1) {
					cout << (abs(x1 - x2) + abs(y1 - y2)) << "\n";
					continue;
				}
				if (count(y, min(y1, y2), max(y1, y2), -k) > 0) {
					cout << (abs(x1 - x2) + abs(y1 - y2)) << "\n";
					continue;
				}
				bool f = false;
				for (int i = 1; i < m; ++i) {
					if (min(y1, y2) - i >= 0) {
						if (get(y, min(y1, y2) - i) + k <= 0) {
							cout << (abs(x1 - x2) + abs(y1 - y2)) + 2 * i << "\n";
							f = true;
							break;
						}
					}
					if (max(y1, y2) + i < m) {
						if (get(y, max(y1, y2) + i) + k <= 0) {
							cout << (abs(x1 - x2) + abs(y1 - y2)) + 2 * i << "\n";
							f = true;
							break;
						}
					}
				}
				if (f) {
					continue;
				}
			}
			if (get(y, y1) + k <= 0 && get(y, y2) + k <= 0) {
				if (count(y, min(y1, y2), max(y1, y2), -k) == abs(y1 - y2) + 1) {
					cout << (abs(x1 - x2) + abs(y1 - y2)) << "\n";
					continue;
				}
				if (count(x, min(x1, x2), max(x1, x2), -k) > 0) {
					cout << (abs(x1 - x2) + abs(y1 - y2)) << "\n";
					continue;
				}
				bool f = false;
				for (int i = 1; i <= n; ++i) {
					if (min(x1, x2) - i >= 0) {
						if (get(x, min(x1, x2) - i) + k <= 0) {
							cout << (abs(x1 - x2) + abs(y1 - y2)) + 2 * i << "\n";
							f = true;
							break;
						}
					}
					if (max(x1, x2) + i <= n) {
						if (get(x, max(x1, x2) + i) + k <= 0) {
							cout << (abs(x1 - x2) + abs(y1 - y2)) + 2 * i << "\n";
							f = true;
							break;
						}
					}
				}
				if (f) {
					continue;
				}
			}
			cout << -1 << "\n";
		}
	}
	return 0;
}
