#include <iostream>
#include <fstream>
#include <vector>
#include <set>
#include <map>
#include <bitset>
#include <iomanip>
#include <numeric>
#include <queue>
#include <deque>
#include <string>
#include <algorithm>
#include <cstring>
#include <cassert>
#include <random>
#define ll long long
#define ld long double
#define null NULL
#define all(a) a.begin(), a.end()
#define rall(a) a.rbegin(), a.rend()

using namespace std;

template<class iterator> void output(iterator begin, iterator end, ostream& out = cerr) {
	while (begin != end) {
		out << (*begin) << " ";
		begin++;
	}
	out << endl;
}

template<class T> void output(T &x, ostream &out = cerr) {
	output(all(x), out);
}

template<class T> int chkmin(T &a, const T &b) {
	if (b < a) {
		a = b;
		return 1;
	}
	return 0;
}

template<class T> int chkmax(T &a, const T &b) {
	if (b > a) {
		a = b;
		return 1;
	}
	return 0;
}

void fast_io() {
	ios_base::sync_with_stdio(0);
	cin.tie(0);
	cout.tie(0);
}

const ll INF = 1e18;

struct segment {
	int l, r;
	segment(): l(0), r(0) {}
	segment(int _l, int _r): l(_l), r(_r) {}
};

bool operator < (segment a, segment b) {
	return a.l < b.l;
}

ll dist(int x, segment &s) {
	if (x >= s.l && x <= s.r) {
		return 0;
	}
	else {
		return min(abs(s.l - x), abs(s.r - x));
	}
}

int n, m, k;
vector<segment> s0, s;
vector<vector<int> > guys, a;

void read_s0() {
	s0.resize(k);
	for (int i = 0; i < k; ++i) {
		cin >> s0[i].l >> s0[i].r;
	}
}

void remove_nested() {
	sort(all(s0));
	int r = 0;
	for (int i = 0; i < k; ++i) {
		if (s0[i].r > r) {
			s.push_back(s0[i]);
			r = s0[i].r;
		}
	}
	k = s.size();
}

void read_guys() {
	guys.resize(m, vector<int> (2 * n));
	for (int i = 0; i < m; ++i) {
		for (int j = 0; j < 2 * n; ++j) {
			cin >> guys[i][j];
		}
	}
}

void make_a() {
	for (int i = 0; i < m; ++i) {
		sort(all(guys[i]));
	}
	a.resize(n);
	for (int i = 0; i < n; ++i) {
		for (int j = 0; j < m; ++j) {
			a[i].push_back(guys[j][2 * i]);
			a[i].push_back(guys[j][2 * i + 1]);
		}
		sort(all(a[i]));
	}
}

ll ans;

ll distance(int pack_id, int seg_id) {
	ll res = 0;
	for (auto val : a[pack_id]) {
		res += dist(val, s[seg_id]);
	}
	return res;
}

void nmk_solution() {
	for (int i = 0; i < n; ++i) {
		ll best = INF;
		for (int x = 0; x < k; ++x) {
			ll cur = distance(i, x);
			chkmin(best, cur);
		}
		ans += best;
	}
}

void mk_solution() {
	int ptr = 0;
	for (int i = 0; i < n; ++i) {
		ll best = distance(i, ptr);
		while (ptr < k - 1) {
			ll new_best = distance(i, ptr + 1);
			if (chkmin(best, new_best)) {
				ptr++;
			}
			else {
				break;
			}
		}
		ans += best;
	}
}

void nk_solution() {
	for (int i = 0; i < n; ++i) {
		ll best = INF;
		ll coords_sum = 0;
		for (auto val : a[i]) {
			coords_sum += (ll)val;
		}
		int cnt1 = 0, cnt2 = 0;
		ll s1 = 0, s2 = 0;
		for (int j = 0; j < k; ++j) {
			while (cnt1 < 2 * m && a[i][cnt1] < s[j].l) {
				s1 += a[i][cnt1++];
			}
			while (cnt2 < 2 * m && a[i][cnt2] < s[j].r) {
				s2 += a[i][cnt2++];
			}
			ll cur_res = ((ll)cnt1 * s[j].l - s1) + (coords_sum - s2 - (ll)(2 * m - cnt2) * s[j].r);
			chkmin(best, cur_res);
		}
		ans += best;
	}
}

void divide(int l, int r, int sl, int sr) {
	if (l > r) {
		return;
	}
	int pos = l + rand() % (r - l + 1), spos;
	ll best = INF;
	for (int i = sl; i <= sr; ++i) {
		ll cur = distance(pos, i);
		if (chkmin(best, cur)) {
			spos = i;
		}
	}
	ans += best;
	divide(l, pos - 1, sl, spos);
	divide(pos + 1, r, spos, sr);
}

void go_qsort() {
	divide(0, n - 1, 0, k - 1);
}

void search_stuff() {
	for (int i = 0; i < n; ++i) {
		ll best = INF;
		int l = 0, r = k - 1;
		while (r - l > 10) {
			int m1 = (2 * l + r) / 3;
			int m2 = (l + 2 * r) / 3;
			ll d1 = distance(i, m1);
			ll d2 = distance(i, m2);
			chkmin(best, d1);
			chkmin(best, d2);
			if (d1 < d2) {
				r = m2;
			}
			else {
				l = m1;
			}
		}
		for (int j = l; j <= r; ++j) {
			ll d = distance(i, j);
			chkmin(best, d);
		}
		ans += best;
	}
}

int main() {
	fast_io();
	cin >> m >> n >> k;
	read_s0();
	remove_nested();
	read_guys();
	make_a();
	go_qsort();
	cout << ans << endl;
}
